/*********************************************************************/
/**
    @file ansi_term.h

    Ansi Terminal Control Codes And Strings

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/

extern const char clrScrnString[];           //lint !e85
extern const char saveCursorString[];        //lint !e85
extern const char unsaveCursorString[];      //lint !e85
extern const char resetScreenString[];       //lint !e85
extern const char scrollUpString[];          //lint !e85
extern const char moveRightString[];         //lint !e85

//** ASCII Special Character Definitions **

#define ASCII_SOH     0x01    // Start of header.
#define ASCII_STX     0x02    // Start of text.
#define ASCII_EOT     0x04    // End of transmission.
#define ASCII_ENQ     0x05    // Is any body out there?
#define ASCII_ACK     0x06    // Message acknowledged.
#define ASCII_NAK     0x15    // Message acknowledged.
#define ASCII_CAN     0x18    // Cancel.
#define ASCII_ESC     0x1B    // Escape.
#define ASCII_DC1     0x11    // Device control 1
#define ASCII_DC2     0x12    // Device control 2
#define ASCII_DC3     0x13    // Device control 3
#define ASCII_CR      0x0D    // Carrige return
#define ASCII_LF      0x0A    // Line feed.

#define ASCII_XON     ASCII_DC1
#define ASCII_XOFF    ASCII_DC3

#define ANSI_MAX_ROWS        25
#define ANSI_MAX_COLS        80

void ANSI_ClearScreen(void (*pStream)(char c));

void ANSI_MoveCursor(void (*pStream)(char c),
                            unsigned row, unsigned col);

void ANSI_SetScrollingRegion(void (*pStream)(char c),
                            unsigned start, unsigned end);

void ANSI_ResetScreen(void (*pStream)(char c));

#define ANSI_SaveCursor(x)  /**/ \
                APPrint_String((x), saveCursorString)

#define ANSI_UnsaveCursor(x)  /**/ \
                APPrint_String((x), unsaveCursorString)

    // Move cursor 1 position right
void ANSI_MoveCursor1Right(void (*pStream)(char c));

    // Move cursor n positions right
void ANSI_MoveCursorNRight(void (*pStream)(char c), unsigned n);

    // Move cursor 1 position left
void ANSI_MoveCursor1Left(void (*pStream)(char c));

    // Move cursor n positions left
void ANSI_MoveCursorNLeft(void (*pStream)(char c), unsigned n);
