/*********************************************************************/
/**
    @file apalloc.h
    Dynamic Memory Allocation for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/


//*********************************************************************

//      *** PUBLIC FUNCTION PROTOTYPES ***

//lint -esym(683, malloc, free)

#define FREE(x)     MEM_Free((void *)(x))
#define malloc(x)   MEM_Alloc(x)
#define free(x)     MEM_Free((void *)(x))

    /** Initialize Memory Allocator Module.

    @return Nothing.

    @context Should be called by startup code, before calling
    Apex_Start().
    */
void MEM_Reset(void);

    /** Allocate a memory block.

    @param blockSize is the desired allocation amount in units
    of bytes.

    @return A pointer to the first usable uint8_t of the allocated
    block, or NULL if the heap is empty and no blocks may be
    allocated.

    @context May be called by startup code, tasks, or interrupt
    routines.
    */
void *MEM_Alloc(uint32_t size);

    /** Free a previously allocated memory block.

    @param pMem is a pointer to a memory block that was previously
    allocated by a call to MEM_Alloc() or malloc().

    @context May be called by startup code, tasks, or interrupt
    routines.
    */

void MEM_Free(void *pMem);

extern const struct TAG_VARIABLE_TABLE memallocVarTab[];
