/***************************************************************
*	aplib.h: Standard Library Functions									*
*															    		*
    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

***************************************************************/

//lint -esym(534,APStrCpy)	// OK to ignore return value.
//lint -esym(534,APMemCpy)	// OK to ignore return value.

//		*** DATA STRUCTURE DEFINITIONS ***


//		*** GLOBAL CONSTANTS ***

extern const char clrScrn[];			// Clear screen.

//extern const dword dwordMaskTable[];	// Bit masks
//extern const word wordMaskTable[];
//extern const uint8_t byteMaskTable[];

//		*** PUBLIC FUNCTION PROTOTYPES ***

/** Copy a string

	Copies a null-terminated string from one location to
	another.  Copies the null too.

	@param pDest: A pointer to the destination of the copy.
	@param pSrc: A pointer to the source string.
	@return Returns a pointer to the destination string. (That's
		silly, I know, but it's C library standard.)
*/
char *APStrCpy(char *pDest, const char *pSrc);

/** Copy memory

	Copies memory bytes from one location to another for a specified
	length.

	@param pDest: A pointer to the destination memory block.
	@param pSrc: A pointer to the source memory block.
	@param length: An integer specifying the length of the
		memory block to copy.
	@return Returns a pointer to the destination memory. (That's
		silly, I know, but it's C library standard.)
*/
void *APMemCpy(void *pDest, const void *pSrc, uint32_t length);

/** Calculate the length of a string.

	This function searches a string for the terminating null, and
	returns an integer specifying the string length in bytes,
	not including the terminating null character.

	@param pSrc: A pointer to the string to be measured.
	@return An integer specifying the length of the string not
	including the terminating null.
*/
int APStrLen(const char *pSrc);
int APStrCmp(const char *pDest, const char *pSrc);
bool APIsDigit(char c);
uint32_t APDecimalShiftLeft(uint32_t numBuf, char c);

#define strcpy(x,y)		APStrCpy((x),(y))
#define strlen(x)			APStrLen(x)
#define memcpy(x,y,z)	APMemCpy((x),(y),(z));
#define strcmp(x,y)		APStrCmp((x),(y))
#define isdigit(x)		APIsDigit(x)

//word UDiv16by8(word dividend, uint8_t divisor);


