//**************************************************************
/**

    @file apparse.h

    API for a General Purpose String Parser.
    (Developed in the context of the PSE-702 project)

    @author ALP

    @version Version 1.0

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************/

/** APParse:

    @param pS:      A pointer to the null terminated string to be
                        parsed.

    @param pCount:  A pointer to an unsigned int buffer into which
                        shall be placed the count of characters actually
                        parsed.

    @param pF:  A null terminated format string containing
                        symbols (specified below) indicating the format
                        of the fields to parse:

    @param ...  A variable parameter list containing pointers to
                    buffers for the items specified in  pFmt.

    @par Format String Codes:
        In the descriptions that follow, the symbol {n} stands for
        an integer parameter that must be substituted in the format
        code.

        @oo %d: Decimal unsigned integer, max 9 digits.

        @oo %{n}f Long integer with up to {n} digits to
            the right of the decimal point.
            0 <= {n} <= 8.  {n} must be specified.

        @oo %c:     Character
        @oo %{n}s: Quoted string, max length {n} chars. {n}
                        must be specified.  Max {n} = 99.
                        The length {n} includes space for
                        the terminating null.
        @oo %{n}w:  Word string delimited by whitespace,
                        max length {n} chars. {n} must be
                        specified.  Max {n} = 99. {n} includes
                        the terminating null.
        @oo %b:     Boolean (0 or 1)
        @oo %{n}x:  Hex integer, max 8 digits.

        @oo space A space in the format string matches zero or
                        more whitespace characters.

        @oo | Must match the null in the pS string.


        @note Types 'd' and 'f' may be up to 9 decimal
            digits long.  In type 'f' there must be at least one
            digit to the left of the decimal point, as in
            0.12345678.  Up to 8 digits may appear to the right
            of the decimal point.

        @par
            All other characters appearing in the format string
            *pFmt must be matched by characters in *pS.

        @return The number of fields properly parsed.  is returned.
            If the returned value is less than the number of
            parameters in *pF, it indicates missing parameters at the
            end of *pS.  If 0 is returned it indicates that an error
            occured while in the process of parsing a field.
*/

uint32_t APParse(const char *pS, unsigned *pCount, char *pF, ...);
