/******************************************************************//**
    @file astream.c

    Audio Processing Stream Interrupt for PSE-702 - API

    @defgroup astream Audio Stream Interrupt

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/
//      *** INCLUDE FILES ***

#include <stdint.h>
#include <stdbool.h>        //lint -esym(766, stdbool.h)

#include "std.h"
#include "tp_application.h" //lint -esym(766, tp_application.h)
#include "lpc213x.h"        //lint -esym(766, lpc213x.h)
#include "target.h"
#include "apex.h"
#include "apalloc.h"
#include "app.h"
#include "io_codec.h"
#include "ssm_satd.h"
//#include "satr.h"
#include "filter.h"
#include "dtmfdet.h"
#include "voice.h"
#include "fgen.h"
#include "astream.h"
// #include "eep.h"

    // Headers used during debug:

#include "apprint.h"        //lint -esym(766, apprint.h)
#include "io_debug.h"       //lint -esym(766, io_debug.h)
#include "io_main.h"        //lint -esym(766, io_main.h)
#include "io_map.h"         //lint -esym(766, io_map.h)

//***************************************************************
//          *** MACROS ***
//***************************************************************

//***************************************************************
//          *** CONSTANTS IN FLASH ***
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//              DB Conversion Table
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

/// The following table is used to select the final gain for
/// each transmitter.  The signal is multiplied by the multiplier
/// factor in the table below, and is then divided by 65536
/// (by downshifting 16).

static const uint32_t dbTable[] =
{
//  Multi-               dB
//  plier        Index  Gain     Magnitude
//  =====        =====  ====     ==========
    2621,       //   0  -28.0    40.000E-03
    2770,       //   1  -27.5    42.271E-03
    2928,       //   2  -27.0    44.670E-03
    3094,       //   3  -26.5    47.206E-03
    3269,       //   4  -26.0    49.886E-03
    3455,       //   5  -25.6    52.718E-03
    3651,       //   6  -25.1    55.710E-03
    3858,       //   7  -24.6    58.873E-03
    4077,       //   8  -24.1    62.215E-03
    4309,       //   9  -23.6    65.747E-03
    4553,       //  10  -23.2    69.479E-03
    4812,       //  11  -22.7    73.423E-03
    5085,       //  12  -22.2    77.591E-03
    5374,       //  13  -21.7    81.996E-03
    5679,       //  14  -21.2    86.651E-03
    6001,       //  15  -20.8    91.570E-03
    6342,       //  16  -20.3    96.768E-03
    6702,       //  17  -19.8   102.261E-03
    7082,       //  18  -19.3   108.066E-03
    7484,       //  19  -18.8   114.201E-03
    7909,       //  20  -18.4   120.684E-03
    8358,       //  21  -17.9   127.534E-03
    8833,       //  22  -17.4   134.774E-03
    9334,       //  23  -16.9   142.425E-03
    9864,       //  24  -16.4   150.510E-03
    10424,      //  25  -16.0   159.054E-03
    11016,      //  26  -15.5   168.083E-03
    11641,      //  27  -15.0   177.625E-03
    12302,      //  28  -14.5   187.708E-03
    13000,      //  29  -14.1   198.364E-03
    13738,      //  30  -13.6   209.624E-03
    14518,      //  31  -13.1   221.524E-03
    15342,      //  32  -12.6   234.100E-03
    16213,      //  33  -12.1   247.389E-03
    17133,      //  34  -11.7   261.433E-03
    18106,      //  35  -11.2   276.273E-03
    19134,      //  36  -10.7   291.957E-03
    20220,      //  37  -10.2   308.530E-03
    21368,      //  38   -9.7   326.045E-03
    22581,      //  39   -9.3   344.553E-03
    23862,      //  40   -8.8   364.113E-03
    25217,      //  41   -8.3   384.783E-03
    26649,      //  42   -7.8   406.626E-03
    28161,      //  43   -7.3   429.709E-03
    29760,      //  44   -6.9   454.102E-03
    31449,      //  45   -6.4   479.880E-03
    33235,      //  46   -5.9   507.122E-03
    35121,      //  47   -5.4   535.910E-03
    37115,      //  48   -4.9   566.332E-03
    39222,      //  49   -4.5   598.481E-03
    41449,      //  50   -4.0   632.456E-03
    43802,      //  51   -3.5   668.358E-03
    46288,      //  52   -3.0   706.299E-03
    48916,      //  53   -2.5   746.394E-03
    51692,      //  54   -2.1   788.765E-03
    54627,      //  55   -1.6   833.541E-03
    57728,      //  56   -1.1   880.859E-03
    61005,      //  57   -0.6   930.863E-03
    64468,      //  58   -0.1   983.706E-03
    68128,      //  59    0.3     1.040E+00
    71995,      //  60    0.8     1.099E+00
    76082,      //  61    1.3     1.161E+00
    80401,      //  62    1.8     1.227E+00
    84965,      //  63    2.3     1.296E+00
    89789,      //  64    2.7     1.370E+00
    94886,      //  65    3.2     1.448E+00
    100272,     //  66    3.7     1.530E+00
    105964,     //  67    4.2     1.617E+00
    111980,     //  68    4.7     1.709E+00
    118336,     //  69    5.1     1.806E+00
    125054,     //  70    5.6     1.908E+00
    132153,     //  71    6.1     2.016E+00
    139655,     //  72    6.6     2.131E+00
    147583,     //  73    7.1     2.252E+00
    155961,     //  74    7.5     2.380E+00
    164814,     //  75    8.0     2.515E+00
    174170,     //  76    8.5     2.658E+00
    184058,     //  77    9.0     2.808E+00
    194506,     //  78    9.4     2.968E+00
    205548,     //  79    9.9     3.136E+00
    217216,     //  80   10.4     3.314E+00
    229547,     //  81   10.9     3.503E+00
    242578,     //  82   11.4     3.701E+00
    256348,     //  83   11.8     3.912E+00
    270900,     //  84   12.3     4.134E+00
    286279,     //  85   12.8     4.368E+00
    302530,     //  86   13.3     4.616E+00
    319704,     //  87   13.8     4.878E+00
    337852,     //  88   14.2     5.155E+00
    357031,     //  89   14.7     5.448E+00
    377299,     //  90   15.2     5.757E+00
    398718,     //  91   15.7     6.084E+00
    421352,     //  92   16.2     6.429E+00
    445271,     //  93   16.6     6.794E+00
    470548,     //  94   17.1     7.180E+00
    497259,     //  95   17.6     7.588E+00
    525487,     //  96   18.1     8.018E+00
    555318,     //  97   18.6     8.473E+00
    586842,     //  98   19.0     8.954E+00
    620155,     //  99   19.5     9.463E+00
    655360      // 100   20.0    10.000E+00
};

//***************************************************************
//              Signals
//***************************************************************

DEFINE_SIGNAL(ASTREAM_DestMessage)
DEFINE_SIGNAL(ASTREAM_SrcPreempted)
DEFINE_SIGNAL(ASTREAM_DestUnassigned)

//***************************************************************
//                   *** VARIABLES ***
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    /// Signal Sources

static int16_t sources[ASTREAM_MAX_SSIG];
static int16_t (*pDeemphasisFilter[MAX_RXCHAN])(
                        enum TAG_RX_CHAN chan, int16_t sample);

static int16_t (*pCtcssRejectionFilter[MAX_RXCHAN])(
                        enum TAG_RX_CHAN chan, int16_t sample);

    /// Signal Destinations

static struct TAG_DEST_STRUCT
{
    int32_t destSums;   ///< Accumulates samples.
    uint32_t finalGain; ///< Final gain for each sample.
    LINK paths;         ///< List of paths for this dest.
    int8_t listed;      ///< Counts times dest is listed in path list.
    bool mute;          ///< Mute this destination if true.

} destinations[ASTREAM_MAX_DSIG];

static uint16_t mPathID;       ///< Number used to identify paths

static uint32_t rmsSquared1;    ///< RMS squared of channel 1.
static uint32_t rmsSquared2;    ///< RMS squared of channel 2.

//***************************************************************
//              *** STATIC FUNCTION PROTOTYPES ****
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

static unsigned DecodePriority(char priority);
static int16_t NullFilter(
                    enum TAG_RX_CHAN unused, int16_t sample);

static int16_t DeemphasisFilter(
                    enum TAG_RX_CHAN chan, int16_t sample);

static int16_t CtcssRejectionFilter1(
                    enum TAG_RX_CHAN chan, int16_t sample);

static int16_t CtcssRejectionFilter2(
                    enum TAG_RX_CHAN chan, int16_t sample);

//***************************************************************
//          *** CONSTANTS IN FLASH ***
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

//***************************************************************
//                      *** API ***
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void ASTREAM_Reset(void)
{
    astream_destsig_t i;

    for(i=ASTREAM_DSIG_TX1; i<ASTREAM_MAX_DSIG; i++)
    {
        destinations[i].mute = false;
        destinations[i].listed = 0;
        Apex_InitLink(&destinations[i].paths, NULL);
    }

    pDeemphasisFilter[RX_CHAN_1] = NullFilter;
    pDeemphasisFilter[RX_CHAN_2] = NullFilter;

    pCtcssRejectionFilter[RX_CHAN_1] = NullFilter;
    pCtcssRejectionFilter[RX_CHAN_2] = NullFilter;

    ASTREAM_UnsubscribeAll();

    mPathID = 1;
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#if 0
void ASTREAM_Start(void)
{
    void ASTREAM_SetDeemphasisFilterIndex(
                                    enum TAG_RX_CHAN chan, int index)

    void ASTREAM_SetCtcssRejectionFilterIndex(
                                    enum TAG_RX_CHAN chan, int index)
}
#endif

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void ASTREAM_UnsubscribeAll(void)
{
    Unsubscribe_ASTREAM_DestMessage();
    Unsubscribe_ASTREAM_DestUnassigned();
    Unsubscribe_ASTREAM_SrcPreempted();
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Select deemphasis or flat filter for Rx channel:

void ASTREAM_SetDeemphasisFilterIndex(
                            enum TAG_RX_CHAN chan, unsigned index)
{
    ASSERT(VALID_RX_CHAN(chan));

    switch(index)
    {
        case 0:
        pDeemphasisFilter[chan] = NullFilter;
        break;

        case 1:
        pDeemphasisFilter[chan] = DeemphasisFilter;
        break;

        default:
        FATAL();
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Select CTCSS rejection filter for Rx channel:

void ASTREAM_SetCtcssRejectionFilterIndex(
                            enum TAG_RX_CHAN chan, unsigned index)
{
    ASSERT(VALID_RX_CHAN(chan));

    if(index)
    {
        switch(index)
        {
            default:
            case 0:
                pCtcssRejectionFilter[chan] = NullFilter;
                break;

            case 1:
                pCtcssRejectionFilter[chan] = CtcssRejectionFilter1;
                break;

            case 2:
                pCtcssRejectionFilter[chan] = CtcssRejectionFilter2;
                break;
        }
    }
    else
        pCtcssRejectionFilter[chan] = NullFilter;
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void ASTREAM_Sample_ISR(void)
{
    long long accum;    // Accumulator for final gain calculation
    astream_path_t *pPath;
    LINK *pNext;
    int16_t sample;
    astream_destsig_t i;
    bool mixing;

    // Get a sample from the codec:
    sample = CDX_GetSample1();

    // Compute the running RMS squared of the samples averaged
    // over 256 samples (approximately 21.85 ms at 11718.75 Hz
    // sampling rate).

    rmsSquared1 = (rmsSquared1 - (rmsSquared1 >> 8))
        + (uint32_t)((sample * sample) >> 8);

    // Give the sample to the subaudible tone decoder:
    SATD_SampleSinkChan1_ISR(sample);

    // Feed DTMF decoder chan 1
    DTMF_SampleSink_ISR(DTMF_CHAN_1, sample);

    // Deemphasize sample if deemphasis is selected
    sample =
        (*pDeemphasisFilter[RX_CHAN_1])(RX_CHAN_1, sample);

    // Remove subaudible tone from the sample:
    sample =
        (*pCtcssRejectionFilter[RX_CHAN_1])(RX_CHAN_1, sample);

    // Save the filtered sample
    sources[ASTREAM_SSIG_RX1] = sample;

    // Get a sample from the codec:
    sample = CDX_GetSample2();

    // Compute the running RMS squared of the samples averaged
    // over 256 samples (approximately 21.85 ms at 11718.75 Hz
    // sampling rate).

    rmsSquared2 = (rmsSquared2 - (rmsSquared2 >> 8))
        + (uint32_t)((sample * sample) >> 8);

    // Give the sample to the subaudible tone decoder:
    SATD_SampleSinkChan2_ISR(sample);

    // Feed DTMF decoder chan 2
    DTMF_SampleSink_ISR(DTMF_CHAN_2, sample);

    // Deemphasize sample if deemphasis is selected
    sample =
        (*pDeemphasisFilter[RX_CHAN_2])(RX_CHAN_2, sample);

    // Remove subaudible tone from the sample:
    sample =
        (*pCtcssRejectionFilter[RX_CHAN_2])(RX_CHAN_2, sample);

    // Save the filtered sample
    sources[ASTREAM_SSIG_RX2] = sample;

    // Get samples from the voice file sources:
    sources[ASTREAM_SSIG_VOICE1] =
                                VOI_GetSample_ISR(VOI_CHAN_1);

    sources[ASTREAM_SSIG_VOICE2] =
                                VOI_GetSample_ISR(VOI_CHAN_2);

    // Assign the samples for each destination.

    for(i=ASTREAM_DSIG_TX1; i<ASTREAM_MAX_DSIG; i++)
    {
        destinations[i].destSums = 0;
        mixing = false;

        // Point to first path in list.
        pNext = destinations[i].paths.pFLink;

        // If the path list is empty, nothing to do.
        while((pPath = pNext->pObject) != NULL)
        {
            // If the path is muted, nothing to do.
            if(!pPath->mute)
            {
                // If the highest priority is medium, sum as
                // many sources as their are paths with
                // medium priority.
                if(pPath->priority == 'M')
                {
                    mixing = true;
                    destinations[i].destSums +=
                                        sources[pPath->srcSig];
                }

                // If the highest priority path for this destination is
                // not medium, then take the source sample and do next
                // destination.
                else if(!mixing)
                {
                    destinations[i].destSums =
                                        sources[pPath->srcSig];
                    break; // while(pPath)
                }

                // Here, we were mixing and ran into a path that is
                // not 'M'.
                else
                    break; // while(path)

            } // if(mute)

            // Now look at the next path on the list:
            pNext = pPath->link.pFLink;

        }   // while(pPath)
    }   // for(i)

    // Add in the CW tones (CW cannot be muted):
    destinations[ASTREAM_DSIG_TX1].destSums +=
                                        FGen_SourceChan2_ISR();

    destinations[ASTREAM_DSIG_TX2].destSums +=
                                        FGen_SourceChan3_ISR();

    // Write the voice file destinations.
    VOI_PutSample_ISR(VOI_CHAN_1,
        CLIP2INT16(destinations[ASTREAM_DSIG_VOICE1].destSums));
    VOI_PutSample_ISR(VOI_CHAN_2,
        CLIP2INT16(destinations[ASTREAM_DSIG_VOICE2].destSums));

    // Apply final gain, clip sample, and write to codec.
    accum = (long long)destinations[ASTREAM_DSIG_TX1].destSums;
    accum *= destinations[ASTREAM_DSIG_TX1].finalGain;
    accum = accum >> 16;
    CDX_PutSample1(CLIP2INT16(accum));

    accum = (long long)destinations[ASTREAM_DSIG_TX2].destSums;
    accum *= destinations[ASTREAM_DSIG_TX2].finalGain;
    accum = accum >> 16;
    CDX_PutSample2(CLIP2INT16(accum));
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Get a new path ID number.  Never return a path ID of 0
     * because this is a reserved value used to indicate an
     * unassigned path.
     */
uint16_t ASTREAM_GetPathID(void)
{
    return(mPathID++);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Get a running average that approximates the sum of the
     * last 256 squared samples.  The square root of this is
     * the approximate RMS of the last 22 ms of the signal.
     */
uint32_t ASTREAM_GetRmsSquared(astream_srcsig_t srcSig)
{
    uint32_t rmsSquared;

    switch(srcSig)
    {
        // Receiver channel 1
        case ASTREAM_SSIG_RX1:
            DI();
            rmsSquared = rmsSquared1;
            EI();
            break;

        // Receiver channel 2
        case ASTREAM_SSIG_RX2:
            DI();
            rmsSquared = rmsSquared2;
            EI();
            break;

        default:
            rmsSquared = 0;
            break;
    }
    return(rmsSquared);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Add a path to the active list.
     *
     */
void ASTREAM_AddPath(
        uint16_t pathID, astream_srcsig_t srcSig,
                astream_destsig_t destSig, char priority)
{
    astream_path_t *pNewPath, *pPath;
    LINK *pNext;
    pNewPath = malloc(sizeof(astream_path_t));
    ASSERT(pNewPath != NULL);
    Apex_InitLink(&pNewPath->link, pNewPath);

    // Load up the path structure and link it to active paths.

    pNewPath->id = pathID;
    pNewPath->srcSig = srcSig;
    pNewPath->destSig = destSig;
    pNewPath->priority = priority;

    // If the destination is muted, mute all paths added to
    // it, except a path for ID.  This allows the transmitter
    // timeout logic to mute the transmitter but still pass
    // the ID and timeout message.
    if(destinations[destSig].mute)
    {
        // If the destination is muted, only allow an ID
        // path to pass audio.
        if(pNewPath->priority == 'I')
            pNewPath->mute = false;
        else
            pNewPath->mute = true;
    }
    else
    {
        pNewPath->mute = false;
    }

    destinations[destSig].listed++;

    // Find the place in the destination's path list for the
    // new path.

    // Point to first path in list.
    pNext = destinations[destSig].paths.pFLink;

    // Look at the first path on the list.  If it's NULL, no need
    // to notify of preemption.  Just insert the new path.
    if((pPath = pNext->pObject) != NULL)
    {
        // If the path is not NULL, see if we will be preempting and
        // send a signal if so.
        if(DecodePriority(priority) > DecodePriority(pPath->priority))
        {
            EMIT_SIGNAL(
                ASTREAM_SrcPreempted, pPath->srcSig, pPath->id);
        }

        // Now, go through the list of paths and insert the
        // new one in front of the first one with lower priority.

        while((pPath = pNext->pObject) != NULL)
        {
            if(DecodePriority(priority) >
                                    DecodePriority(pPath->priority))
                break;

            else
            {
                // Now look at the next path on the list:
                pNext = pPath->link.pFLink;
            }
        }
    }

    DI();
    Apex_Link(pNext->pBLink, &pNewPath->link);
    EI();
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Remove all paths.
     *
     * This function clears the path list.
     *
     * All paths are dynamically allocated so must be freed.
     */
void ASTREAM_RemoveAllPaths(void)
{
    // Pointer to next path in list.
    LINK *pNext;
    astream_path_t *pPath;
    astream_destsig_t i;

    // Go through all destinations and clear their path lists.
    for(i=ASTREAM_DSIG_TX1; i<ASTREAM_MAX_DSIG; i++)
    {
        pNext = destinations[i].paths.pFLink;

        // Loop through the paths and remove inactive ones.
        while((pPath = pNext->pObject) != NULL)
        {
            DI();
            Apex_Unlink(&pPath->link);
            EI();

            // If we are removing the last path for a device, send
            // a signal.
            ASSERT(destinations[pPath->destSig].listed >= 0);

            if(--destinations[pPath->destSig].listed == 0)
            {
                EMIT_SIGNAL(ASTREAM_DestUnassigned, pPath->destSig, 0);
            }

            free(pPath);    // De-allocate the path

            // Now look at the next path on the list:
            pNext = pPath->link.pFLink;
        }
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Remove all paths that match a particular path ID
     * number.
     *
     * This function searches through the active path list
     * and removes all paths that match the specified
     * ID number.
     *
     * All paths are dynamically allocated so must be freed.
     */
void ASTREAM_RemovePathByID(uint16_t pathID)
{
    // Pointer to next path in list.
    LINK *pNext;
    astream_path_t *pPath;
    astream_destsig_t i;

    for(i=ASTREAM_DSIG_TX1; i<ASTREAM_MAX_DSIG; i++)
    {
        pNext = destinations[i].paths.pFLink;

        // Loop through the paths and remove inactive ones.
        while((pPath = pNext->pObject) != NULL)
        {
            if(pPath->id == pathID)
            {
                DI();
                Apex_Unlink(&pPath->link);
                EI();

                // If we are removing the last path for a device, send
                // a signal.
                ASSERT(destinations[pPath->destSig].listed >= 0);

                if(--destinations[pPath->destSig].listed == 0)
                {
                    EMIT_SIGNAL(
                            ASTREAM_DestUnassigned, pPath->destSig, 0);
                }

                free(pPath);    // De-allocate the path
            }

            // Now look at the next path on the list:
            pNext = pPath->link.pFLink;
        }
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Remove all paths that match a particular destination
     * device.
     *
     * This function searches through the active path list
     * and removes all paths that match the specified
     * destination device.
     *
     * This function is used when a transmitter channel
     * times out, to remove all audio paths attached to
     * that transmitter, for example.
     *
     * All paths are dynamically allocated so must be freed.
     */
void ASTREAM_RemovePathByDestDev(astream_destsig_t destSig)
{
    // Pointer to next path in list.
    LINK *pNext;
    astream_path_t *pPath;

    pNext = destinations[destSig].paths.pFLink;

    // Loop through the paths and remove them.
    while((pPath = pNext->pObject) != NULL)
    {
        DI();
        Apex_Unlink(&pPath->link);
        EI();

        // If we are removing the last path for a device, send
        // a signal.
        ASSERT(destinations[destSig].listed >= 0);

        if(--destinations[destSig].listed == 0)
        {
            EMIT_SIGNAL(ASTREAM_DestUnassigned, destSig, 0);
        }

        // Now look at the next path on the list:
        pNext = pPath->link.pFLink;

        free(pPath);    // De-allocate the path
    }

    // The path count for this signal must now be zero, otherwise
    // we lost count somewhere along the way.
    ASSERT(destinations[destSig].listed == 0);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Signal all destination devices assigned to a
     * particular path ID.
     */
void ASTREAM_MessageDestDevicesByPathID(
                        uint16_t pathID, astream_msg_t *pMsg)
{
    // Pointer to next path in list.
    LINK *pNext;
    astream_path_t *pPath;
    astream_destsig_t i;

    for(i=ASTREAM_DSIG_TX1; i<ASTREAM_MAX_DSIG; i++)
    {
        // Point to first path in list.
        pNext = destinations[i].paths.pFLink;

        // Loop through the paths.
        while((pPath = pNext->pObject) != NULL)
        {
            if(pPath->id == pathID)
            {
                EMIT_SIGNAL(ASTREAM_DestMessage,
                                pPath->destSig, (uintptr_t)pMsg);
            }
            // Now look at the next path on the list:
            pNext = pPath->link.pFLink;
        }
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Signal all destination devices currently active due
     * to the current path priorities.
     */
void ASTREAM_MessageDestDevicesByActivePath(
                        uint16_t pathID, astream_msg_t *pMsg)
{
    // Pointer to next path in list.
    LINK *pNext;
    astream_path_t *pPath;
    astream_destsig_t i;

    for(i=ASTREAM_DSIG_TX1; i<ASTREAM_MAX_DSIG; i++)
    {
        // Point to first path in list.
        pNext = destinations[i].paths.pFLink;

        if((pPath = pNext->pObject) != NULL)
        {
            if(pPath->id == pathID)
            {
                EMIT_SIGNAL(ASTREAM_DestMessage,
                            pPath->destSig, (uintptr_t)pMsg);
            }

            if(pPath->priority == 'M')
            {
                // Loop through medium priority paths.
                while((pPath = pNext->pObject) != NULL)
                {
                    if(pPath->priority == 'M')
                    {
                        if(pPath->id == pathID)
                        {
                            EMIT_SIGNAL(ASTREAM_DestMessage,
                                pPath->destSig, (uintptr_t)pMsg);
                        }
                        // Now look at the next path on the list:
                        pNext = pPath->link.pFLink;
                    }
                    else
                        break; // while

                } // while
            } // if priority == 'M'
        } // if pObject != NULL

    } // for ASTREAM dest devices
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Mute destination device.
     */
void ASTREAM_MuteDestDev(astream_destsig_t destSig)
{
    // Pointer to next path in list.
    LINK *pNext;
    astream_path_t *pPath;

    DI(); destinations[destSig].mute = true; EI();

    pNext = destinations[destSig].paths.pFLink;

    // Loop through the paths and remove inactive ones.
    while((pPath = pNext->pObject) != NULL)
    {
        DI(); pPath->mute = true; EI();
        // Now look at the next path on the list:
        pNext = pPath->link.pFLink;
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /* Unute destination device.
     */
void ASTREAM_UnmuteDestDev(astream_destsig_t destSig)
{
    // Pointer to next path in list.
    LINK *pNext;
    astream_path_t *pPath;

    //    destinations[destSig].mute = false;

    pNext = destinations[destSig].paths.pFLink;

    // Loop through the paths and remove inactive ones.
    while((pPath = pNext->pObject) != NULL)
    {
        DI(); pPath->mute = false; EI();
        // Now look at the next path on the list:
        pNext = pPath->link.pFLink;
    }

    DI(); destinations[destSig].mute = false; EI();
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

LINK *ASTREAM_GetDestPathList(astream_destsig_t destSig)
{
    return(&destinations[destSig].paths);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Levels range from 0 to 100:

void ASTREAM_SetTx1Level(uint8_t index)
{
    destinations[TX_CHAN_1].finalGain = dbTable[index];
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

// Levels range from 0 to 100:

void ASTREAM_SetTx2Level(uint8_t index)
{
    destinations[TX_CHAN_2].finalGain = dbTable[index];
}

//***************************************************************
//      Static Functions
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Decode a priority letter into a number:

static unsigned DecodePriority(char priority)
{
    switch(priority)
    {
        default:
        case 'I': return(0);    // Lowest priority
        case 'L': return(1);    // Low priority
        case 'M': return(2);    // Medium priority
        case 'H': return(3);    // High priority
    }
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      First Order Deemphasis Filters
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

int16_t ASTREAM_TestFilter(int16_t sample)
{
    static const biquad_coeff_t
        hipassCoefficients = { 11, 1909, -3719, 1869, 0, 0 };

    static const biquad_coeff_t
            notchCoefficients = { 11, 1451, -2709, 996, -4024, 0 };

    static biquad_delay_t highpassDelays[MAX_RXCHAN];

    static biquad_delay_t notchDelays[MAX_RXCHAN];

    // Prefilter the sample to remove freqs below about 500 Hz:

    sample = FILT_2ndOrderNotch(
                    &notchCoefficients, &notchDelays[0], sample);

    sample = FILT_2ndOrderHighPass(
                &hipassCoefficients, &highpassDelays[0], sample);

    return(sample);       // Straight through
//    return(DeemphasisFilter(RX_CHAN_1, sample));
//    return(CtcssRejectionFilter1(RX_CHAN_1, sample));
//    return(CtcssRejectionFilter2(RX_CHAN_1, sample));
}

    /// Null filter: Filter returns same value as was presented.

static int16_t NullFilter(
                    enum TAG_RX_CHAN unused, int16_t sample)
{
    return(sample);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // Deemphasis Filter:

    // F0=100 Hz, Fs=11718.75, Z0= -0.2, Gain normalized to 0dB
    // at 178 Hz.  Gain is 6 dB at 0 Hz.  Gain is approx. -3.8 dB
    // at 300 Hz.  Gain from 300 Hz to 3000 Hz is -20 dB.

static int16_t DeemphasisFilter(
                    enum TAG_RX_CHAN chan, int16_t sample)
{
    static const first_order_coeff_t
                    coefficients = { 13, 728, -7764, 1638 };

    static first_order_delay_t
                        delays[MAX_RXCHAN] = { {0, 0}, {0, 0} };

    return(FILT_1stOrderGeneralized(
                        &coefficients, &delays[chan], sample));
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                  CTCSS Rejection Filters
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // CTCSS Rejection Filter 1:

    // Third order Butterworth high-pass filter.  Intended to mimic
    // the response of the PSE-508 rejection filter.

    // Fs=11718.75, F0=300 Hz.

static int16_t CtcssRejectionFilter1(
                    enum TAG_RX_CHAN chan, int16_t sample)
{
    static const first_order_coeff_t
        firstOrderCoefficients = { 13, 7764, -6975, 0 };

    static const biquad_coeff_t
        secondOrderCoefficients = { 13, 7359, -14971, 6975, 0, 0 };

    static first_order_delay_t
                firstOrderDelays[MAX_RXCHAN] = { {0, 0}, {0, 0} };

    static biquad_delay_t
        secondOrderDelays[MAX_RXCHAN] = { {0, 0, 0, 0}, {0, 0, 0, 0} };

    int16_t intermediateResult;

    intermediateResult = FILT_2ndOrderHighPass(
        &secondOrderCoefficients, &secondOrderDelays[chan], sample);

    return(FILT_1stOrderHighPass(&firstOrderCoefficients,
                        &firstOrderDelays[chan], intermediateResult));
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // CTCSS Rejection Filter 2:

    // Fourth order composite high-pass filter.  This filter consists
    // of a 2nd order notch followed by a 2nd order high-pass.  The
    // filter provides superior attenuation of CTCSS frequencies
    // above 100.0 Hz at the expense of some low frequency audio.

    // Fs=11718.75,
    // First biquad: Highpass, F0=310 Hz, Q=4.5, double zeros at
    //      0 Hz (DC).
    // Second biquad: Notch, F0=300 Hz, Q=0.67, double zeros at
    //      223 Hz.
    // Gain adjusted to 0 dB at Nyquist frequency.

static int16_t CtcssRejectionFilter2(
                        enum TAG_RX_CHAN chan, int16_t sample)
{
    static const biquad_coeff_t
        hipassCoefficients = { 11, 1997, -3965, 1973, 0, 0 };

    static const biquad_coeff_t
            notchCoefficients = { 11, 1719, -3376, 1428, -4067, 0 };

    static biquad_delay_t
        highpassDelays[MAX_RXCHAN] = { {0, 0, 0, 0}, {0, 0, 0, 0} };

    static biquad_delay_t
        notchDelays[MAX_RXCHAN] = { {0, 0, 0, 0}, {0, 0, 0, 0} };

    int16_t intermediateResult;

    intermediateResult = FILT_2ndOrderNotch(
                    &notchCoefficients, &notchDelays[chan], sample);

    return(FILT_2ndOrderHighPass(&hipassCoefficients,
                        &highpassDelays[chan], intermediateResult));
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                           END
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
