/******************************************************************//**

    @file audiosm.h

    Audio State Machine for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/
//               *** MODULE ID DEFINITIONS ****
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#define AUD_ID(x)   (MODULE(MID_AUD) + (x))

#define TID_AUD     AUD_ID(1)   // Audio state machine task ID

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                  Data Types
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    /// This type is defined to simplify passing parameters with
    /// the AUD_StartMorse signal.

typedef struct TAG_AUD_MORSE_PARMS
{
    char *pString;
    uint8_t level;

} aud_morse_parms_t;

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                  SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

DECLARE_SIGNAL(AUD_GeneratedAudioOff)
DECLARE_SIGNAL(AUD_StartMorse)

DECLARE_SIGNAL(AUD_IDAudioStarted)

DECLARE_SIGNAL(AUD_IDAudioComplete)

DECLARE_SIGNAL(AUD_MessageAudioStarted)
DECLARE_SIGNAL(AUD_MessageAudioComplete)

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *//**
    Playback Audio File

    Send signal to transmitter channel 1 message slot.

        @oo slotParm1 transmitter channel number to play ID on.
        @oo slotParm2 File number to send to destination device.
    */
DECLARE_SIGNAL(AUD_VoiceID)

DECLARE_SIGNAL(AUD_CancelPlayback)

DECLARE_SIGNAL(AUD_Announce)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      *** PUBLIC FUNCTION PROTOTYPES ***
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void AUD_Reset(void);
void AUD_Start(void);

void AUD_UnsubscribeAll(void);
void AUD_AddAnno(unsigned index);
void AUD_DeleteAnno(unsigned index);
void AUD_AddAllAnno(void);
void AUD_DeleteAllAnno(void);

void AUD_StartAudioJobs(enum TAG_TX_CHAN txChan);

void AUD_PlayAnno(unsigned short index);
void AUD_PlayCB(enum TAG_TX_CHAN chan, eep_courtsey_beep_t *pBeep);
void AUD_PlayFile(enum TAG_TX_CHAN chan, unsigned short fileNum);

void AUD_NotifyTxTimeout(enum TAG_TX_CHAN chan);
void AUD_NotifyTxDisable(enum TAG_TX_CHAN chan);
void AUD_NotifyUserAudioOn(enum TAG_TX_CHAN txChan);
void AUD_NotifyUserAudioOff(enum TAG_TX_CHAN chan);

void AUD_NotifySourcePreempted(enum TAG_TX_CHAN txChan);

void AUD_NotifyTxReady(enum TAG_TX_CHAN txChan);

void AUD_NotifyMorseComplete(enum TAG_TX_CHAN txChan);
void AUD_NotifyFlashComplete(enum TAG_TX_CHAN txChan);
void AUD_NotifyTxTimeoutCleared(enum TAG_TX_CHAN chan);

void AUD_ScheduleID(enum TAG_TX_CHAN txChan);
void AUD_ScheduleOpeningID(enum TAG_TX_CHAN txChan);
void AUD_ScheduleOK(enum TAG_TX_CHAN txChan);
void AUD_ScheduleERR(enum TAG_TX_CHAN txChan);
void AUD_ScheduleAnnouncement(enum TAG_TX_CHAN txChan);

void AUD_UnscheduleAll(enum TAG_TX_CHAN txChan);

extern const struct TAG_VARIABLE_TABLE audVarTab[];

