/****************************************************************//**

    @file central.h
    Central Control Task for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************/

//  *** Central Module Number and Related Definitions ***

#define CEN_ID(x)   (MODULE(MID_CEN) + (x))

#define TID_CEN     CEN_ID(1)   ///< Central task ID

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void CEN_Reset(void);
void CEN_SubscribeAll(void);
void CEN_UnsubscribeAll(void);

void CEN_SetConsoleToUserContext(void);
void CEN_SetConsoleToTestContext(void);

    // SUSI can call this to reset the DTMF paths.
void CEN_AssignDTMFPaths(void);

void CEN_RecordAudioFile(enum TAG_RX_CHAN chan, unsigned file);

void CEN_CancelRecording(void);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Tell central that the CTCSS encoder has completed a
    // reverse burst.

void CEN_SATE_Complete(enum TAG_TX_CHAN chan);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      Functions called by SUSI and DUSI
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void CEN_PlayID(enum TAG_TX_CHAN chan);
void CEN_PlayAnno(unsigned short index);
void CEN_PlayFile(enum TAG_TX_CHAN chan, unsigned short fileNum);

    /** SUSI and DUSI call this to tell central to disable a
        transmitter.

        @param txChan: Transmitter channel number.
    */
void CEN_TxDisable(enum TAG_TX_CHAN txChan);

    /** SUSI and DUSI call this to tell central to enable a
        transmitter.

        @param txChan: Transmitter channel number.
    */
void CEN_TxEnable(enum TAG_TX_CHAN txChan);


    /** SUSI and DUSI call this to tell central to enable or
        disable a receiver.

        @param rxChan: Receiver channel number.
        @param flag: True to enable the receiver, false to
            disable it.
    */
void CEN_RxEnable(enum TAG_RX_CHAN rxChan, bool flag);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//   Functions called by Transmitter State Machine (txsm)
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    /** TXSM calls this to Tell central that a transmitter is
        turning on.

        @param txChan: Transmitter channel number.
    */
void CEN_TxOn(enum TAG_TX_CHAN txChan);

    /** TXSM calls this to Tell central that a transmitter is
        turning off.

        @param txChan: Transmitter channel number.
    */
void CEN_TxOff(enum TAG_TX_CHAN txChan);

    /** TXSM calls this to Tell central that a transmitter has
        timed out.

        @param txChan: Transmitter channel number.
    */
void CEN_TxTimeout(enum TAG_TX_CHAN txChan);

    /** TXSM calls this to Tell central to start any generated
        audio jobs that might be scheduled.

        @param txChan: Transmitter channel number.
    */
void CEN_StartGeneratedAudio(enum TAG_TX_CHAN txChan);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      Functions called by Audio State Machine (audiosm)
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#if 0
    /** Signal CENTRAL that ID audio is being generated.

        @param chan: Transmitter channel number.
    */
void CEN_SignalIDAudioStarted(enum TAG_TX_CHAN chan);

    /** Signal CENTRAL that an ID has been completed.

        @param txChan: Transmitter channel number.
    */
void CEN_SignalIDAudioComplete(enum TAG_TX_CHAN txChan);

    /** Signal CENTRAL that message audio is being generated.

        @param chan: Transmitter channel number.
    */
void CEN_SignalMessageAudioStarted(enum TAG_TX_CHAN chan);

    /** Signal CENTRAL that message audio has been
        completed.

        @param chan: Transmitter channel number.
    */

void CEN_SignalMessageAudioComplete(enum TAG_TX_CHAN chan);

    /** Signal CENTRAL that all generated audio is
        completed.

        @param chan: Transmitter channel number.
    */
void CEN_GeneratedAudioOff(enum TAG_TX_CHAN chan);
#endif

//void CEN_ScheduleDTMFOK(enum TAG_RX_CHAN chan);
//void CEN_ScheduleDTMFERR(enum TAG_RX_CHAN chan);
void CEN_ScheduleAnno(enum TAG_TX_CHAN chan);
void CEN_ScheduleCB(enum TAG_TX_CHAN chan);
//void CEN_ScheduleTxTimeout(enum TAG_TX_CHAN chan);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void CEN_MorseEncode(enum TAG_TX_CHAN chan, char *pString);
void CEN_MorseComplete(enum TAG_TX_CHAN chan);
void CEN_MorseWhisper(enum TAG_TX_CHAN chan);
void CEN_MorseShout(enum TAG_TX_CHAN chan);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void CEN_SaveXMPayload(const char *pBuff);  // Save XModem payload.

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // Functions to support test tone output.

void CEN_RequestTestTone(void);
void CEN_TestToneOff(void);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

extern const struct TAG_VARIABLE_TABLE cenVarTab[];


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                              END
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
