/****************************************************************//**

    @file console.h
    User Input Shell for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************/

#define CON_ID(x)   (MODULE(MID_CON) + (x))

#define TID_CON     CON_ID(1)       // Console task 1 ID

    /** Command table entry definition.
     *
     * This structure represents a single entry in a command table.
     */
typedef struct TAG_CMD_TABLE
{
    char *pName;                    ///< Name of command
    uint8_t (*pCommand)(char *pLine);  ///< Function to perform command
    char *pDescription;             ///< Command description
    char *pSyntax;                  ///< Command syntax help
    char *pLongSyntax;              ///< Command parameter help

} console_cmd_tbl_t;

typedef struct TAG_CONSOLE_CONTEXT
{
    console_cmd_tbl_t *cmdTable;
    void (*pStream)(char c);
    void (*pStateMachine)(void);
    char prompt;
} console_context_t;

#if 0
//      Character Received Event Definition
//      Zero can be used here because module ID 0 is reserved and
//      will not be used by other modules.

 /// This event means a character was received

#define EV_CON_RXDATA   (MODULE(MID_CON)+0)
#endif

    // Parser Error Messages

#define CON_STS_OK          0   ///< "OK",
#define CON_STS_SYNTAX      1   ///< "Syntax error.",
#define CON_STS_RANGE       2   ///< "Parameter out of range."
#define CON_STS_NULL        3   ///< No message
#define CON_STS_STATECHG    4   ///< No msg or prompt. (State change)
#define CON_STS_NOTFOUND    5   ///< Command not found.

//      *** PUBLIC FUNCTION PROTOTYPES ***

void Con_Reset(void);
void Con_Start(void);

void Con_RxChar_ISR(char c);    // Send a character to the console.

void Con_DBPrintCon_ISR(void);

void Con_SetContext(const console_cmd_tbl_t *pNewContext,
        char newPrompt, void (*pNewStream)(char c),
        void (*pNewStateMachine)(void));

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
/** Send a message to the Console mailbox

    This function dynamically allocates and sends a message to the
    console mailbox.  It should be called by console context modules
    to send messages, since the mailbox that is monitored by the
    console resides in console.c and is not otherwise accessible by
    the context modules.

    @param id: A user defined integer identifying the type of message.

    @param data1: A pointer sized integer which may contain any
        data the user wishes to send.

    @param data2: A pointer sized integer which may contain any
        data the user wishes to send.

    @return Nothing
*/

void Con_SendMsgExt(uint16_t id, uintptr_t data1, uintptr_t data2);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
/** Initialize a timer to send a message to the Console mailbox
    on timeout.

    This function initializes a user allocated TMR structure before
    its first use.  The timer will send a message to the Console
    mailbox upon timeout.  It should be called by console context
    modules to initialize timers since the Console mailbox is not
    otherwise accessible.

    This function does not start the timer, it only initializes
    the timer's structure.  The timer should be started by calling
    Apex_StartTimer().

    @param pTmr: A pointer to the timer to be initialized.

    @param id: The id of the message to send when the timer times out.

    @param data1: A pointer sized integer which may contain any
        data the user wishes to send.

    @param data2: A pointer sized integer which may contain any
        data the user wishes to send.

    @return Nothing
*/

void Con_InitMsgTimer(
        TMR *pTmr, uint16_t id, uintptr_t data1, uintptr_t data2);

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void Con_Echo(bool state);
void Con_CharacterSink(char c);
void Con_Prompt(void);

extern const struct TAG_VARIABLE_TABLE consoleVarTab[];
