/*********************************************************************/
/**
    @file filter.c
    Miscellaneous Audio Filters for PSE-702

    @defgroup filter Miscellaneous Digital Audio Filters

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

***************************************************************/
//      *** INCLUDE FILES ***

#include <stdint.h>
#include <stdbool.h>        //lint -esym(766, stdbool.h)

#include "std.h"
//#include "tp_system.h"      //lint -esym(766, tp_system.h)
#include "tp_application.h" //lint -esym(766, tp_application.h)
#include "lpc213x.h"        //lint -esym(766, lpc213x.h)
#include "target.h"
#include "apex.h"
#include "esalaw.h"         //lint -esym(766, esalaw.h)
#include "app.h"

#include "io_map.h"         //lint -esym(766, io_map.h)
//#include "io_codec.h"
#include "astream.h"
#include "eep.h"            //lint -esym(766, eep.h)

#include "filter.h"

    // Headers used during debug:

#include "apprint.h"        //lint -esym(766, apprint.h)
#include "io_debug.h"       //lint -esym(766, io_debug.h)
#include "io_main.h"        //lint -esym(766, io_main.h)

//***************************************************************
//                     *** MACROS ****

//***************************************************************

//***************************************************************
//                   *** VARIABLES ***
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#if 0
    // Array of deemphasis filters, one for each channel.

static struct TAG_FILT_DEEMPH
{
    uint16_t a1;            // Coefficient a1
    uint16_t b0;            // Coefficient b0
    int16_t b1;             // Coefficient b1

    int32_t zz1;            // Delay term storage (zero)
    int32_t zp1;            // Delay term storage (pole)

} deemphFilters[MAX_RXCHAN];
#endif

//***************************************************************
//                       CONSTANTS
//***************************************************************

    // Debugging Table for Variable Display

const struct TAG_VARIABLE_TABLE filtVarTab[] =
{
    {NULL, NULL, VAR_TYPE_NONE, 0}
};

#if 0
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Filter coefficient array

    // Filter coefficient array

    // Coefficient a1 is subtracted in the diagram, so if the
    // Mathcad spreadsheet calculates it as a negative value
    // (which it always does), then it has to be added
    // (subtracting a negative).

    // If this is confusing, see the block diagram in the
    // Mathcad model for deemphasis.

    // I take care of this by listing a1 as a positive number
    // in the table below, and adding it in the actual filter
    // implementation.  This saves a little time over doing
    // signed multiplication.

#define E2_DEEMPH           13      // Coefficient scaling factor

static struct
{
    uint16_t a1;
    uint16_t b0;
    int16_t b1;

} coefficients[MAX_NUM_FILTERS] =
{
    // (Test 3 in Excel file 080610 readings.xls)
    // F0=250 Hz, Fs=11718.75, Z0=0.3, Gain normalized to 0dB
    // at 0 Hz. Gain is approx. -3.8 dB at 300 Hz.
    { 7164, 1469, -441 },

    // The first order filter above does not cut off fast enough
    // and I received complaints about "tinny" audio, so
    // Alternate Filter #1 was designed:
    // F0=100 Hz, Fs=11718.75, Z0= -0.2, Gain normalized to 0dB
    // at 178 Hz.  Gain is 6 dB at 0 Hz.  Gain is approx. -3.8 dB
    // at 300 Hz.

    { 7764, 728, 146 },

    // This one has a zero at the Nyquist frequency.

    // F0=240 Hz, Fs=11718.75, Z0= -1, Gain normalized to 0dB
    // at 0 Hz.  Gain is approx. -4.1 dB at 300 Hz.

    { 7203, 495, 495 }

};
#endif

//***************************************************************
//                      *** API ***
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void FILT_Reset(void)
{
#if 0
    enum TAG_RX_CHAN i;

    for(i=RX_CHAN_1; i<MAX_RXCHAN; i++)
    {
        FILT_SetFilterIndex(i, 0);
        deemphFilters[i].zp1 = 0;
        deemphFilters[i].zz1 = 0;
    }
#endif
}

void FILT_Start(void)
{
#if 0
    enum TAG_RX_CHAN i;

    for(i=RX_CHAN_1; i<MAX_RXCHAN; i++)
    {
        FILT_SetFilterIndex(i, EEP_GetFilterIndex(i));
    }
#endif
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // Set up a filter
#if 0
void FILT_SetFilterIndex(
                enum TAG_RX_CHAN rxChan, unsigned i)
{
    deemphFilters[rxChan].a1 = coefficients[i].a1;
    deemphFilters[rxChan].b0 = coefficients[i].b0;
    deemphFilters[rxChan].b1 = coefficients[i].b1;
}
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
#if 0
    // Two Channel, First Order De-emphasis Filter

int16_t FILT_Deemphasis_ISR(
                    enum TAG_RX_CHAN chan, int16_t x_sub_n)
{
    int32_t y_sub_n;

    y_sub_n = (
        ((int32_t)x_sub_n * deemphFilters[chan].b0)
        + (deemphFilters[chan].zz1 * deemphFilters[chan].b1)
        + (deemphFilters[chan].zp1 * deemphFilters[chan].a1)
        ) >> E2_DEEMPH;

    deemphFilters[chan].zz1 = (int32_t)(int32_t)x_sub_n;
    deemphFilters[chan].zp1 = y_sub_n;

    return((int16_t)y_sub_n);
}
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // First Order Low-Pass, Direct Form 1 Implementation

    // In the low pass there is a zero at Z=-1 (Nyquist frequency),
    // so b1 = 1.  b0 is factored out and becomes part of the final
    // gain k.

    // Transfer Function:

    //            k * (Z + 1)
    //          ------------------
    //              Z + a1

    // Tested 6/15/2013 ALP

int16_t FILT_1stOrderLowPass(const first_order_coeff_t *pParms,
                            first_order_delay_t *pDelay, int16_t x)
{
    int32_t y;

    y = (
            (
                ((int32_t)x + pDelay->zz) * pParms->k
            )
            - (pDelay->zp * pParms->a)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz = (int32_t)x;
    pDelay->zp = y;

    return((int16_t)y);
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // First Order High-Pass, Direct Form 1 Implementation

    // In the high pass there is a zero at Z=1 (DC), so
    // b1 = -1.  b0 is factored out and becomes part of the final
    // gain k.

    // Transfer Function:

    //            k * (Z - 1)
    //          ------------------
    //               Z + a

    // Tested 6/15/2013 ALP

int16_t FILT_1stOrderHighPass(const first_order_coeff_t *pParms,
                            first_order_delay_t *pDelay, int16_t x)

{
    int32_t y;

    y = (
            (
                ((int32_t)x - pDelay->zz) * pParms->k
            )
            - (pDelay->zp * pParms->a)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz = (int32_t)x;
    pDelay->zp = y;

    return((int16_t)y);
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // First Order Generalized, Direct Form 1 Implementation

    // In the generalized first order filter, no assumptions are
    // made about the value or sign of any coefficients.  There
    // will be a zero at -b1 and a pole at -a1.

    // Transfer Function:

    //            k * (Z + b1)
    //          ------------------
    //              Z + a1

int16_t FILT_1stOrderGeneralized(const first_order_coeff_t *pParms,
                                first_order_delay_t *pDelay, int16_t x)

{
    int32_t y;

    y = (
            (
                (int32_t)x +
                (
                    (pDelay->zz * pParms->b)
                    >> pParms->scale_factor
                )
            ) * pParms->k
            - (pDelay->zp * pParms->a)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz = (int32_t)x;
    pDelay->zp = y;

    return((int16_t)y);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // 2nd Order Notch, Direct Form 1 Implementation

    // In the notch, b1 is formed by the sum of the complex
    // conjugate zeros, b2 is formed by the product of the
    // complex conjugate zeros and is therefore equal to +1 since
    // the zeros are on the unit circle (which is where they
    // should be for a notch filter), and b0 is factored out and
    // becomes part of the final gain k.

    // All signs are the same as used in the biquad diargam in the
    // Mathcad analysis files.  Note that the signs in the denominator
    // of the transfer function below are +, but in the implementation
    // the the value of the 'a' terms are subtracted.  This is
    // correct and is according to the diagram.

    // Transfer Function:

    //                2
    //          k * (Z  + b1*Z + 1)
    //          --------------------
    //              2
    //             Z  + a1*Z + a2

    // Tested 6/15/2013 ALP

int16_t FILT_2ndOrderNotch(
    const biquad_coeff_t *pParms, biquad_delay_t *pDelay, int16_t x)
{
    int32_t y;

    // Calculate the output:

    y = (
            (
                (
                    (int32_t)x +
                    (
                        (pDelay->zz1 * pParms->b1)
                        >> pParms->scale_factor
                    ) + pDelay->zz2
                ) * pParms->k
            )
            - (pDelay->zp1 * pParms->a1)
            - (pDelay->zp2 * pParms->a2)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz2 = pDelay->zz1;
    pDelay->zz1 = (int32_t)x;
    pDelay->zp2 = pDelay->zp1;
    pDelay->zp1 = y;

    return((int16_t)y);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // 2nd Order High Pass, Direct Form 1 Implementation

    // In the high-pass filter realization, There is a double-zero
    // at Z = +1.  b0 is factored of the biquad transfer function
    // and becomes part of the final gain term leaving b1 = -2 and
    // b2 = +1.  This saves a couple multiplications.

    // All signs are the same as used in the biquad diargam in the
    // Mathcad analysis files.  Note that the signs in the denominator
    // of the transfer function below are +, but in the implementation
    // the the value of the 'a' terms are subtracted.  This is
    // correct and is according to the diagram.

    // Transfer Function:

    //                2
    //          k * (Z  - 2*Z + 1)
    //          --------------------
    //              2
    //             Z  + a1*Z + a2

    // Tested 6/15/2013 ALP

int16_t FILT_2ndOrderHighPass(
    const biquad_coeff_t *pParms, biquad_delay_t *pDelay, int16_t x)
{
    int32_t y;

    // Calculate the output:

    y = (
            (
                (
                    (int32_t)x
                    - (pDelay->zz1 << 1)
                    + pDelay->zz2
                ) * pParms->k
            )
            - (pDelay->zp1 * pParms->a1)
            - (pDelay->zp2 * pParms->a2)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz2 = pDelay->zz1;
    pDelay->zz1 = (int32_t)x;
    pDelay->zp2 = pDelay->zp1;
    pDelay->zp1 = y;

    return((int16_t)y);
}


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // 2nd Order Low Pass, Direct Form 1 Implementation

    // In the low-pass filter realization, There is a double-zero
    // at Z = -1.  b0 is factored of the biquad transfer function
    // and becomes part of the final gain term leaving b1 = +2 and
    // b2 = +1.  This saves a couple multiplications.

    // All signs are the same as used in the biquad diargam in the
    // Mathcad analysis files.  Note that the signs in the denominator
    // of the transfer function below are +, but in the implementation
    // the the value of the 'a' terms are subtracted.  This is
    // correct and is according to the diagram.

    // Transfer Function:

    //                2
    //          k * (Z  + 2*Z + 1)
    //          --------------------
    //              2
    //             Z  + a1*Z + a2

int16_t FILT_2ndOrderLowPass(
    const biquad_coeff_t *pParms, biquad_delay_t *pDelay, int16_t x)
{
    int32_t y;

    // Calculate the output:

    y = (
            (
                (
                    (int32_t)x
                    + (pDelay->zz1 << 1)
                    + pDelay->zz2
                ) * pParms->k
            )
            - (pDelay->zp1 * pParms->a1)
            - (pDelay->zp2 * pParms->a2)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz2 = pDelay->zz1;
    pDelay->zz1 = (int32_t)x;
    pDelay->zp2 = pDelay->zp1;
    pDelay->zp1 = y;

    return((int16_t)y);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // 2nd Order Band Pass, Direct Form 1 Implementation

    // In the band-pass filter realization, There is a zero
    // at Z = -1 and at Z = +1.  b0 is factored of the biquad
    // transfer function and becomes part of the final gain
    // term leaving b1 = 0 and b2 = -1.  This saves
    // a couple multiplications.

    // All signs are the same as used in the biquad diargam in the
    // Mathcad analysis files.  Note that the signs in the denominator
    // of the transfer function below are +, but in the implementation
    // the the value of the 'a' terms are subtracted.  This is
    // correct and is according to the diagram.

    // Transfer Function:

    //                   2
    //             k * (Z  - 1)
    //          --------------------
    //              2
    //             Z  + a1*Z + a2

int16_t FILT_2ndOrderBandPass(
    const biquad_coeff_t *pParms, biquad_delay_t *pDelay, int16_t x)
{
    int32_t y;

    // Calculate the output:

    y = (
            (
                ( (int32_t)x + pDelay->zz2 ) * pParms->k
            )
            - (pDelay->zp1 * pParms->a1)
            - (pDelay->zp2 * pParms->a2)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz2 = pDelay->zz1;
    pDelay->zz1 = (int32_t)x;
    pDelay->zp2 = pDelay->zp1;
    pDelay->zp1 = y;

    return((int16_t)y);
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    // 2nd Order Generalized Biquad, Direct Form 1 Implementation

    // In the generalized biquad, all parameters are used exactly
    // as in the Mathcad diagram.  No assumptions about coefficient
    // sign or value are made.  b0 is factored out of the numerator
    // and becomes part of the final gain k.

    // The generalized biquad can create responses equivalent to
    // all of the above functions, but is less efficient because
    // no assumptions are made about coefficients or signs.

    // All signs are the same as used in the biquad diargam in the
    // Mathcad analysis files.  Note that the signs in the denominator
    // of the transfer function below are +, but in the implementation
    // the the value of the 'a' terms are subtracted.  This is
    // correct and is according to the diagram.

    // Transfer Function:

    //                2
    //          k * (Z  + b1*Z + b2)
    //          --------------------
    //              2
    //             Z  + a1*Z + a2

int16_t FILT_2ndOrderGeneralized(
    const biquad_coeff_t *pParms, biquad_delay_t *pDelay, int16_t x)
{
    int32_t y;

    // Calculate the output:

    y = (
            (
                (
                    (int32_t)x +
                    (
                        (
                              (pDelay->zz1 * pParms->b1)
                            + (pDelay->zz2 * pParms->b2)
                        )   >> pParms->scale_factor
                    )
                ) * pParms->k
            )
            - (pDelay->zp1 * pParms->a1)
            - (pDelay->zp2 * pParms->a2)
        ) >> pParms->scale_factor;

    // Shift the delay terms:
    pDelay->zz2 = pDelay->zz1;
    pDelay->zz1 = (int32_t)x;
    pDelay->zp2 = pDelay->zp1;
    pDelay->zp1 = y;

    return((int16_t)y);
}

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                           END
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
