/*********************************************************************/
/**
    @file io_codec.h

    API for Driver for ARM SSI device as used in PSE-702 to
    communicate with the AD73322L codec.

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************/

    /// Codec sample rate in Hz:

#define CDX_SAMPLE_RATE_HZ      11718.75

    /// The following definition is used elsewhere in \#if statements
    /// to choose parameters for filters etc.  Conditional statements
    /// in the pre-processor must not use floating point constants,
    /// so we round it to an integer. (It is never actually used
    /// in calculations, only as a parameter selector).

    /// Possible values of CDX_PARAMETER_FREQ_SEL are:
    ///      9375    (For CDX_SAMPLE_RATE_HZ of 9375.0 Hz)
    ///      11719   (For CDX_SAMPLE_RATE_HZ of 11718.75 Hz)

#define CDX_PARAMETER_FREQ_SEL     11719


    /// Codec 0 dB Reference
    ///
    /// This level is the number of counts corresponding to 5000 Hz
    /// deviation, taking into account the max and min codec voltage
    /// reference values over temperature, etc.  (This is a 'peak'
    /// measurement).  See the Mathcad spreadsheet entitled
    /// "Codec Voltage Levels.mcd" under Models/Schematic.

#define CDX_0DB_REFERENCE   26810

    // Codec channel IDs:

enum TAG_CDX_CHAN
{
    CDX_CHAN_1 = 0,     ///< Codec Channel 1
    CDX_CHAN_2,         ///< Codec Channel 2
    MAX_CDX_CHAN        ///< Codec Number of channels
};

    // Convert an Rx channel number to a CDX channel number.

#define RXCHAN_2_CDXCHAN(x)     ((enum TAG_CDX_CHAN)(x))
#define TXCHAN_2_CDXCHAN(x)     ((enum TAG_CDX_CHAN)(x))
#define NUM_2_CDXCHAN(x)        ((enum TAG_CDX_CHAN)((x)-1)

#if 0
/// @name Subscription number assignments for the audio filters:
///{
#define CDX_SLOT_SATD           0   ///< CTCSS decoder
#define CDX_SLOT_SATR           1   ///< CTCSS suppression filter
#define CDX_SLOT_DTMFDET        2   ///< DTMF decoder
#define CDX_SLOT_DEMPH          3   ///< Deemphasis filter
#define CDX_SLOT_VOICE_REC      4   ///< Voice recorder module
#define CDX_SLOT_XPOINT         5   ///< Crosspoint switch
//#define                       6
#define CDX_SLOT_FGEN           7   ///< Tone generator
#define CDX_SLOT_VOICE_PLAY     8   ///< Voice playback module
///@}
#endif

//      *** PUBLIC FUNCTION PROTOTYPES ***

void CDX_Reset(void);                   // Call after reset.
void CDX_Start(void);                   // Call after Apex startup.


    /** This function sets an ISR vector to process codec samples.
     *
     */

void CDX_Subscribe(void (*pAudioSampleTickHandlerFunction)(void));

void CDX_Unsubscribe(void);

// void CDX_ProcessSamples_ISR(void);  // Called by io_codec.c
// void CDX_InitializeFilterVectors(enum TAG_CDX_CHAN chan);

int16_t CDX_GetSample1(void);
int16_t CDX_GetSample2(void);

void CDX_PutSample1(int16_t sample);
void CDX_PutSample2(int16_t sample);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                            END
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

