/****************************************************************//**

    @file io_debug.h

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************/

    // Convienient cut-paste lines:

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void IO_DB_Reset(void);
void IO_DisplayMemoryBytes(
    void (*pStream)(char c), const uint8_t *pMem, uintptr_t addr, uint32_t length);

void IO_DisplayMemoryDwords(void (*pStream)(char c),
        const unsigned *pMem, uintptr_t addr, uint32_t length);
void IO_DBCrashMonitor(uintptr_t stackPointer);

    // Debug UART Macros:
//lint -emacro(506, PROBE)
//lint -emacro(774, PROBE)

#define PROBE(cond, c)  if(cond) IO_DBPrintChar(c)
#define IO_DBRxChar()   IO_PolledRxUART1()

    // Trace Macros:

#define TRACE1(x)       IO_DBTraceIn((x),1)
#define TRACE2(x)       IO_DBTraceIn((x),2)

    // Functions:

void IO_DB_BreakpointDisarm(void);
void IO_DB_BreakpointArm(void);
bool IO_DB_BreakpointIsArmed(void);
void IO_DB_Breakpoint(void);

void IO_DBPause(char idChar);       // Wait for a key on UART1
void IO_DBPrintChar(char c);        // Print a char on UART1
void IO_DBHalt(char c);
bool IO_DBMark(uint8_t count);

void IO_DBTraceReset(void);
void IO_DBTraceEnable(void);
void IO_DBTraceDisable(void);
void IO_DBTraceOut(uintptr_t *pD, uint32_t *pT);
void IO_DBTraceIn(uintptr_t d, uint32_t t);
uint32_t IO_DBTraceGetCount(void);

void IO_DBPrintDWord(uint32_t data);
void IO_DBPrintWord(uint16_t data);
void IO_DBPrintByte(uint8_t data);
void IO_DBPrintNibble(uint8_t nibble);
void IO_DB_CRLF(void);
void IO_DBPrintString(const char *pC);

void * __builtin_return_address (unsigned int level);
void * __builtin_frame_address (unsigned int level);
