/*********************************************************************/
/**
    @file io_main.h
        Contains I/O definitions for LPC-213X processor for
        the PSE-702 application.

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

***************************************************************/

    // General Purpose I/O:

void IO_GPIO_Reset(void);       // Init I/O pins after reset.

    // UART I/O

void IO_UART0_Reset(void);
void IO_UART1_Reset(void);

    // Stream I/O

#define STDOUT      IO_TxUART0
#if DEBUG_PORT != 0
#define STDERR      IO_PolledTxUART1
#else
#define STDERR      IO_PolledTxUART0
#endif

    // Transmitting to UART 0:

#define TxUserConsole(x)        IO_TxUART0(x)

void IO_TxUART0(char c);
uint8_t IO_PolledRxUART0(void);
void IO_PolledTxUART0(char c);

    // Interrupt Driven Serial I/O Control:

void IO_EnableIntDrivenRxUART0(void);
void IO_EnableIntDrivenTxUART0(void);

void IO_DisableIntDrivenUART0(void);
void Subscribe_IO_RxUart0(void (*pRxCharSink)(char c));
void Unsubscribe_IO_RxUart0(void);

    // I/O for Specific Application Modules:

void IO_InitDTMFADC(void);
void IO_SystemTimer_Start(void);
void IO_WriteFGenSamples(char sample1, char sample2);

void IO_IRQ_Handler(void) __attribute__ ((interrupt("IRQ")));
void IO_FIQ_Handler(void) __attribute__ ((interrupt("FIQ")));
void IO_SWI_Handler(void) __attribute__ ((interrupt("SWI")));
void IO_ABORT_Handler(void) __attribute__ ((interrupt("ABORT")));
void IO_UNDEF_Handler(void) __attribute__ ((interrupt("UNDEF")));

void IO_PolledTxUART1(char c);
uint8_t IO_PolledRxUART1(void);

#define IO_RxUART0()        IO_PolledRxUART0()

    // PWM Definitions:

#define PWM_DI()    VICIntEnClr = BV(VIC_PWM0)  // Disable PWM Ints
#define PWM_EI()    VICIntEnable = BV(VIC_PWM0) // Enable PWM Ints


    // The PWM sample rate is based on the requirement that we
    // sample at 10 times the highest frequency of interest.
    // The PWM is going to be used to generate CTCSS frequencies
    // so the highest frequency being generated is about 250 Hz.

#define PWM_SAMPLE_RATE_HZ      2500.0  // Sample rate in Hz.

typedef int16_t pwm_source_t(void);

    // PWM channel IDs:

enum TAG_PWM_CHAN
{
    PWM_CHAN_1 = 0,     ///< PWM Channel 1: Tx1 Encode
    PWM_CHAN_2,         ///< PWM Channel 2: Tx2 Encode
    PWM_MAX_CHAN        ///< PWM Number of channels
};

void IO_PWM_Reset(void);        // Called after reset.
void IO_PWM_Start(void);        // Called after Apex startup.

void IO_PWM_Subscribe(enum TAG_PWM_CHAN chan, pwm_source_t *pFilter);
void IO_PWM_Unsubscribe(enum TAG_PWM_CHAN chan);

    // Real Time Clock

void IO_RTC_SetTime(uint8_t min, uint8_t hr,
                        uint8_t dom, uint8_t month, uint16_t year);

void IO_RTC_ReadTime(uint8_t *pMin, uint8_t *pHr,
                    uint8_t *pDom, uint8_t *pMonth, uint16_t *pYear);

    // Enable and disable Timer 0 interrupts:

void IO_SystemTimerDI(void);
void IO_SystemTimerEI(void);
