/***************************************************************
*                                                              *
*       io_map_PSE-702.h:                                      *
*                                                              *
*       Contains basic I/O definitions for the PSE-702 PCB.    *
*                                                              *
    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************

    History

    071205 ALP  Modified for PSE-702 PCB V1.0 prototype hardware.

    120329 ALP  Modified for PSE-702 PCB V2.0 prototype hardware.

****************************************************************/

    // Vectored Interrupt Controller Assignments:
    // Slot 0 has highest priority, slot 15 the lowest.

#define VIC_PRI_SSP     VICVectAddr0    // SSP Priority 0 (CODEC)
#define VIC_CTRL_SSP    VICVectCntl0

#define VIC_PRI_SPI0    VICVectAddr1    // SPI Priority 1 (FLASH)
#define VIC_CTRL_SPI0   VICVectCntl1

#define VIC_PRI_TMR0    VICVectAddr2    // Timer0 Priority 2
#define VIC_CTRL_TMR0   VICVectCntl2

#define VIC_PRI_PWM     VICVectAddr5    // PWM Priority 5
#define VIC_CTRL_PWM    VICVectCntl5

#define VIC_PRI_UART0   VICVectAddr8    // UART0 Priority 8
#define VIC_CTRL_UART0  VICVectCntl8

#define VIC_PRI_WDOG    VICVectAddr6    // UART0 Priority 6
#define VIC_CTRL_WDOG   VICVectCntl6

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    //  *** General Purpose I/O Definitions ***


    // Bits     Nibble
    // 0-3      0
    // 4-7      1
    // 8-11     2
    // 12-15        3
    // 16-19        4
    // 20-23        5
    // 24-27        6
    // 28-31        7

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

        // Pin Function Selections:

        // The following define the pin functions that have been
        // selected on the LPC213X.  They are written into PINSEL0
        // and PINSEL1 (the pin select registers for port 0)
        // in io.c.

    // PORT0_FUNC_A Enable: UART0, I2C0, SCI, PWM2, UART1,
    //                              GPIO(10-15), SSI

    // PORT0_FUNC_B Enable: GPIO(16), PWM5, GPIO(22-31)

    //                      3322 2222 2222 1111 1111 1100 0000 0000
    // Bit Num:         1098 7654 3210 9876 5432 1098 7654 3210

    //  PORT0_FUNC_A    0000 0000 0000 0101 1001 0101 0101 0101
    //                         0    0    0    5    9    5    5    5
#define PORT0_FUNC_A        0x00059555  // PINSEL0 (See bits above)

    //  PORT0_FUNC_B    0000 0000 0000 0000 0000 0110 1010 1000
    //                         0    0    0    0    0    6    A    8
#define PORT0_FUNC_B        0x000006A8  // PINSEL1 (See bits above)

    //  PORT1_FUNC      0000 0000 0000 0000 0000 0000 0000 1000
    //                         0    0    0    0    0    0    0    0
//#define PORT1_FUNC        0x00000000  // PINSEL2 (Allow to default)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Port 0 Pin Definitions:

        // P0.0: UART0 TxD
        // P0.1: UART0 RxD
        // P0.2: I2C0 SCL0
        // P0.3: I2C0 SDA0

        // P0.4: SCI SCK
        // P0.5: SCI MISO
        // P0.6: SCI MOSI
        // P0.7: PWM2 (CTCSS Encode 1)

        // P0.8: UART1 TxD
        // P0.9: UART1 RxD

//lint -esym(773, CLEAR_OUTPUT) // Not parenthesized.
//lint -esym(773, SET_OUTPUT)   // Not parenthesized.
//lint -esym(773, O_RESET)  // Not parenthesized.

#define CLEAR_OUTPUT(x,y)   IOCLR ## x = BV(y)
#define SET_OUTPUT(x,y)     IOSET ## x = BV(y)

        // P0.10

#define O_FLASH         10 // P0.10: Flash Select
#define O_FLASH_ENA()   CLEAR_OUTPUT(0,O_FLASH)
#define O_FLASH_DIS()   SET_OUTPUT(0,O_FLASH)

        // P0.11: SCL1

#define O_DPOT2         12
#define O_DPOT2_ENA()   CLEAR_OUTPUT(0,O_DPOT2) // P0.12: Digital POT 2
#define O_DPOT2_DIS()   SET_OUTPUT(0,O_DPOT2)

#define O_DPOT3         13
#define O_DPOT3_ENA()   CLEAR_OUTPUT(0,O_DPOT3) // P0.13: Digital POT 3
#define O_DPOT3_DIS()   SET_OUTPUT(0,O_DPOT3)

        // P0.14: SDA1

#define O_T5            15
#define O_T5_L()        CLEAR_OUTPUT(0,O_T5) // P0.15
#define O_T5_H()        SET_OUTPUT(0,O_T5)

#define O_CODEC         16      // P0.16: Codec Reset/Run
#define O_CODEC_RST()   CLEAR_OUTPUT(0,O_CODEC)
#define O_CODEC_RUN()   SET_OUTPUT(0,O_CODEC)

        // P0.17: SSI CLK (Clock)
        // P0.18: SSI SDI (Serial Data In)
        // P0.19: SSI SDO (Serial Data Out)

        // P0.20: SSI FS (Frame Sync)
        // P0.21: PWM5 (CTCSS Encode 2)

        // P0.22: GPIO:
#define O_DPOT1         22      // P0.22: Digital POT 1
#define O_DPOT1_ENA()   CLEAR_OUTPUT(0,O_DPOT1)
#define O_DPOT1_DIS()   SET_OUTPUT(0,O_DPOT1)

        // P0.23: GPIO:
#define I_RUS1()        ((IOPIN0 & BV(23)) != 0) // P0.23: RUS Channel 1

        // P0.24: (Not Available)

        // P0.25: GPIO (T6):
#define O_FTP0          25      // P0.25: Firmware Test Pt 0
#define O_FTP0_H()      SET_OUTPUT(0,O_FTP0)
#define O_FTP0_L()      CLEAR_OUTPUT(0,O_FTP0)
#define O_FTP0_STROBE() O_FTP0_H(); O_FTP0_L()

        // P0.26: GPIO
#define I_RUS2()        ((IOPIN0 & BV(26)) != 0) // P0.26: RUS Channel 2

        // P0.27: ADC0.0 Input (ADC) (T7)
#define O_FTP2          27      // P0.27: Firmware Test Pt 2
#define O_FTP2_H()      SET_OUTPUT(0,O_FTP2)
#define O_FTP2_L()      CLEAR_OUTPUT(0,O_FTP2)
#define O_FTP2_STROBE() O_FTP2_H(); O_FTP2_L()

        // P0.28: ADC0.1 Input (ADC) (T10)
#define O_FTP3          28      // P0.28: Firmware Test Pt 3
#define O_FTP3_H()      SET_OUTPUT(0,O_FTP3)
#define O_FTP3_L()      CLEAR_OUTPUT(0,O_FTP3)
#define O_FTP3_STROBE() O_FTP3_H(); O_FTP3_L()

        // P0.29: GPIO
#define O_DPOT4         29      // P0.29: Digital POT 4
#define O_DPOT4_ENA()   CLEAR_OUTPUT(0,O_DPOT4)
#define O_DPOT4_DIS()   SET_OUTPUT(0,O_DPOT4)

        // P0.30: GPIO (T9)

        // P0.31: Is output only.  (It's read on power up and if high,
        // JTAG is enabled): (T8)
#define O_FTP1          31      // P0.31: Firmware Test Pt 1
#define O_FTP1_H()      SET_OUTPUT(0,O_FTP1)
#define O_FTP1_L()      CLEAR_OUTPUT(0,O_FTP1)
#define O_FTP1_STROBE() O_FTP1_H(); O_FTP1_L()

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // Port 1 Pin Definitions:

        // P1.16: GPIO

#define O_PTT1LED       16      // PTT1 LED
#define O_PTT1LED_H()   SET_OUTPUT(1,O_PTT1LED)
#define O_PTT1LED_L()   CLEAR_OUTPUT(1,O_PTT1LED)

        // P1.17: GPIO

#define O_PTT2LED       17      // PTT2 LED
#define O_PTT2LED_H()   SET_OUTPUT(1,O_PTT2LED)
#define O_PTT2LED_L()   CLEAR_OUTPUT(1,O_PTT2LED)

        // P1.18: GPIO

#define O_RUS1LED       18      // RUS1 LED
#define O_RUS1LED_H()    SET_OUTPUT(1,O_RUS1LED)
#define O_RUS1LED_L()    CLEAR_OUTPUT(1,O_RUS1LED)

        // P1.19: GPIO

#define O_RUS2LED       19      // RUS2 LED
#define O_RUS2LED_H()   SET_OUTPUT(1,O_RUS2LED)
#define O_RUS2LED_L()   CLEAR_OUTPUT(1,O_RUS2LED)


        // P1.20: Input, pulled high externally to disable trace port.

        // P1.21: GPIO

#define O_T22           21      // Testpoint 22

        // P1.22: GPIO

#define O_T23           22      // Testpoint 23

        // P1.23: GPIO
        // PTT Signals: Channel 1 is active high, controlls MII Tx.
        //              Channel 2 has software defined active level.

#define O_PTT1          23
#define O_PTT1_H()      CLEAR_OUTPUT(1,O_PTT1) // P1.24: PTT Channel 1
#define O_PTT1_L()      SET_OUTPUT(1,O_PTT1)

        // P1.24: GPIO

#define O_PTT2          24
#define O_PTT2_H()      CLEAR_OUTPUT(1,O_PTT2) // P1.24: PTT Channel 2
#define O_PTT2_L()      SET_OUTPUT(1,O_PTT2)

        // P1.25: GPIO

#define O_T24       25      // Testpoint 24

        // P1.26: GPIO
        // P1.27: GPIO

        // P1.28: GPIO
        // P1.29: GPIO
        // P1.30: GPIO
        // P1.31: GPIO

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // GPIO Port Direction and Defaults:

    // Port Direction:  1 bits are outputs, 0 bits are inputs.
    // Port Levels:     0 bits are active high, 1 bits are active low.
    // Port Defaults:       Values written to port after reset.

    // It's not necessary to set these values for ports that are
    // not used for GPIO functons.  Pins not used for GPIO are
    // left as inputs.

    //                  3322 2222 2222 1111 1111 1100 0000 0000
    // Bit Num:         1098 7654 3210 9876 5432 1098 7654 3210

    // To make a port an output, simply put a BV() in the list below
    // with that port's name or number.  All 1 bits in PORT0_DIR
    // will be made outputs.

#define PORT0_DIR           (                         \
                                    BV(O_FLASH)     | \
                                    BV(O_DPOT1)     | \
                                    BV(O_DPOT2)     | \
                                    BV(O_DPOT3)     | \
                                    BV(O_DPOT4)     | \
                                    BV(O_CODEC)     | \
                                    BV(O_FTP0)      | \
                                    BV(O_FTP1)      | \
                                    BV(O_FTP2)      | \
                                    BV(O_FTP3)       \
                                )


    //                  3322 2222 2222 1111 1111 1100 0000 0000
    // Bit Num:         1098 7654 3210 9876 5432 1098 7654 3210

#define PORT0_DEFAULTS  (               \
                                    BV(O_FLASH) | \
                                    BV(O_DPOT2) | \
                                    BV(O_DPOT3) | \
                                    BV(O_DPOT4) | \
                                    BV(O_DPOT1) \
                                )

    //                  3322 2222 2222 1111 1111 1100 0000 0000
    // Bit Num:         1098 7654 3210 9876 5432 1098 7654 3210

#define PORT1_DIR           (                           \
                                    BV(O_PTT1LED)       | \
                                    BV(O_PTT2LED)       | \
                                    BV(O_RUS1LED)       | \
                                    BV(O_RUS2LED)       | \
                                    BV(O_PTT1)          | \
                                    BV(O_PTT2)          \
                            )


    //                  3322 2222 2222 1111 1111 1100 0000 0000
    // Bit Num:         1098 7654 3210 9876 5432 1098 7654 3210

#define PORT1_DEFAULTS      0



