/*********************************************************************/
/**
    @file io_misc.c

    Driver for miscellaneous devices on the PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

    @author ALP

    @version 110916 ALP Cloned from flash driver and io_spi.h

    @defgroup io_misc Miscellaneous I/O Device Driver
    @ingroup io_devices
    @{
**********************************************************************/


//      *** INCLUDE FILES ***

#include <stdint.h>
#include <stdbool.h>

#include "lpc213x.h"        // LPC213x definitions
#include "std.h"
//#include "testpoints.h"
#include "target.h"

#include "io_debug.h"       //lint -esym(766, io_debug.h)

//#include "apex.h"
#include "io_map.h"
#include "io_spi.h"
#include "io_misc.h"


//***************************************************************

//      *** COMPILE SWITCHES ***

//***************************************************************

//      *** MACROS ****

//lint -esym(750, NUM_POTS)   // Not referenced unless debug.

#define NUM_POTS        4       ///< 4 electronic potentiometers

#define IO_DIGITAL_POT_MAX      255 ///< Full scale range of pot.

//***************************************************************

//      *** GENERAL DEFINITIONS ****

//***************************************************************

//      *** VARIABLES ***

#if 0
static uint8_t potLevelImage[NUM_POTS]; // Potentiometer levels

const struct TAG_VARIABLE_TABLE miscVarTab[] =
{
#if TP_ENABLE_VARTABS != 0
    AVARIABLE(potLevelImage),
#endif

    {NULL, NULL, VAR_TYPE_NONE, 0}
};
#endif

//***************************************************************
//              STATIC FUNCTION PROTOTYPES

static void SelectPotRx1(bool select);
static void SelectPotRx2(bool select);
static void SelectPotTx1(bool select);
static void SelectPotTx2(bool select);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                  DIGITAL POTENTIOMETER API
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // The following function sets the level of a potentiometer.

    // This function should be called only from a task.

void IO_SetPotLevel(uint8_t chan, unsigned level)
{
    uint8_t potValue;

    ASSERT(chan < NUM_POTS);
    ASSERT(level <= 100);

    // Convert percent to digital pot units:
    potValue = (uint8_t)(((level * IO_DIGITAL_POT_MAX) + 50) / 100);

    switch(chan)
    {
        case 0:
            SPI_Command(SelectPotRx1, &potValue, 1);
            break;

        case 1:
            SPI_Command(SelectPotRx2, &potValue, 1);
            break;

        case 2:
            SPI_Command(SelectPotTx1, &potValue, 1);
            break;

        case 3:
            SPI_Command(SelectPotTx2, &potValue, 1);
            break;

        default:
            FATAL();
    }
}


    // Select a Potentiometer

static void SelectPotRx1(bool select)
{
    if(select)
        O_DPOT1_ENA();          // Select RX1 pot
    else
        O_DPOT1_DIS();          // Deelect RX1 pot
}

static void SelectPotRx2(bool select)
{
    if(select)
        O_DPOT2_ENA();          // Select RX1 pot
    else
        O_DPOT2_DIS();          // Deelect RX1 pot
}

static void SelectPotTx1(bool select)
{
    if(select)
        O_DPOT3_ENA();          // Select RX1 pot
    else
        O_DPOT3_DIS();          // Deelect RX1 pot
}

static void SelectPotTx2(bool select)
{
    if(select)
        O_DPOT4_ENA();          // Select RX1 pot
    else
        O_DPOT4_DIS();          // Deelect RX1 pot
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
///@}                          END OF FILE
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
