/****************************************************************
*                                                               *
*       lpc213x.h:                                              *
*                                                               *
*       Contains I/O definitions for LPC-213X processor for     *
*       the PSE-702 application.                                *
*                                                               *
    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

*                                                               *
****************************************************************/


    // Vectored Interrupt Controller (VIC)

#define VICIRQStatus   (*((volatile unsigned long *) 0xFFFFF000))
#define VICFIQStatus   (*((volatile unsigned long *) 0xFFFFF004))
#define VICRawIntr     (*((volatile unsigned long *) 0xFFFFF008))
#define VICIntSelect   (*((volatile unsigned long *) 0xFFFFF00C))
#define VICIntEnable   (*((volatile unsigned long *) 0xFFFFF010))
#define VICIntEnClr    (*((volatile unsigned long *) 0xFFFFF014))
#define VICSoftInt     (*((volatile unsigned long *) 0xFFFFF018))
#define VICSoftIntClr  (*((volatile unsigned long *) 0xFFFFF01C))
#define VICProtection  (*((volatile unsigned long *) 0xFFFFF020))
#define VICVectAddr    (*((volatile unsigned long *) 0xFFFFF030))
#define VICDefVectAddr (*((volatile unsigned long *) 0xFFFFF034))

#define VICVectAddr0   (*((volatile unsigned long *) 0xFFFFF100))
#define VICVectAddr1   (*((volatile unsigned long *) 0xFFFFF104))
#define VICVectAddr2   (*((volatile unsigned long *) 0xFFFFF108))
#define VICVectAddr3   (*((volatile unsigned long *) 0xFFFFF10C))
#define VICVectAddr4   (*((volatile unsigned long *) 0xFFFFF110))
#define VICVectAddr5   (*((volatile unsigned long *) 0xFFFFF114))
#define VICVectAddr6   (*((volatile unsigned long *) 0xFFFFF118))
#define VICVectAddr7   (*((volatile unsigned long *) 0xFFFFF11C))
#define VICVectAddr8   (*((volatile unsigned long *) 0xFFFFF120))
#define VICVectAddr9   (*((volatile unsigned long *) 0xFFFFF124))
#define VICVectAddr10  (*((volatile unsigned long *) 0xFFFFF128))
#define VICVectAddr11  (*((volatile unsigned long *) 0xFFFFF12C))
#define VICVectAddr12  (*((volatile unsigned long *) 0xFFFFF130))
#define VICVectAddr13  (*((volatile unsigned long *) 0xFFFFF134))
#define VICVectAddr14  (*((volatile unsigned long *) 0xFFFFF138))
#define VICVectAddr15  (*((volatile unsigned long *) 0xFFFFF13C))

#define VICVectCntl0   (*((volatile unsigned long *) 0xFFFFF200))
#define VICVectCntl1   (*((volatile unsigned long *) 0xFFFFF204))
#define VICVectCntl2   (*((volatile unsigned long *) 0xFFFFF208))
#define VICVectCntl3   (*((volatile unsigned long *) 0xFFFFF20C))
#define VICVectCntl4   (*((volatile unsigned long *) 0xFFFFF210))
#define VICVectCntl5   (*((volatile unsigned long *) 0xFFFFF214))
#define VICVectCntl6   (*((volatile unsigned long *) 0xFFFFF218))
#define VICVectCntl7   (*((volatile unsigned long *) 0xFFFFF21C))
#define VICVectCntl8   (*((volatile unsigned long *) 0xFFFFF220))
#define VICVectCntl9   (*((volatile unsigned long *) 0xFFFFF224))
#define VICVectCntl10  (*((volatile unsigned long *) 0xFFFFF228))
#define VICVectCntl11  (*((volatile unsigned long *) 0xFFFFF22C))
#define VICVectCntl12  (*((volatile unsigned long *) 0xFFFFF230))
#define VICVectCntl13  (*((volatile unsigned long *) 0xFFFFF234))
#define VICVectCntl14  (*((volatile unsigned long *) 0xFFFFF238))
#define VICVectCntl15  (*((volatile unsigned long *) 0xFFFFF23C))

    // Vectored Interrupt Controller Function Select Numbers:
    // When using the registers that require a bit mask, use
    // the BV() macro to obtain the mask.

#define VIC_WDT             0       // Watchdog timer
#define VIC_CORE0           2       // Arm Core 0
#define VIC_CORE1           3       // Arm Core 1
#define VIC_TIMER0          4       // Timer 0
#define VIC_TIMER1          5       // Timer 1
#define VIC_UART0           6       // UART 0
#define VIC_UART1           7       // UART 1
#define VIC_PWM0            8       // PWM 0
#define VIC_I2C0            9       // I2C 0
#define VIC_SPI0            10      // SPI 0
#define VIC_SSP             11      // SPI 1 / SSP
#define VIC_PLL             12      // 12 PLL UNLOCK
#define VIC_RTC             13      // Real Time Clock
#define VIC_EINT0           14      // External Interrupt 0
#define VIC_EINT1           15      // External Interrupt 1
#define VIC_EINT2           16      // External Interrupt 2
#define VIC_EINT3           17      // External Interrupt 3
#define VIC_AD0             18      // A/D Converter 0
#define VIC_I2C1            19      // I2C 1
#define VIC_BOD             20      // Brown-Out Detector
#define VIC_ADC1            22      // A/D Converter 1

    // Other Vectored Interrupt Controller Definitions

#define VIC_IRQ_EN          0x20    // A/D Converter 1


    // Pin Connect Block

#define PINSEL0        (*((volatile unsigned long *) 0xE002C000))
#define PINSEL1        (*((volatile unsigned long *) 0xE002C004))
#define PINSEL2        (*((volatile unsigned long *) 0xE002C014))

    // General Purpose Input/Output (GPIO)

#define IOPIN0         (*((volatile unsigned long *) 0xE0028000))
#define IOSET0         (*((volatile unsigned long *) 0xE0028004))
#define IODIR0         (*((volatile unsigned long *) 0xE0028008))
#define IOCLR0         (*((volatile unsigned long *) 0xE002800C))
#define IOPIN1         (*((volatile unsigned long *) 0xE0028010))
#define IOSET1         (*((volatile unsigned long *) 0xE0028014))
#define IODIR1         (*((volatile unsigned long *) 0xE0028018))
#define IOCLR1         (*((volatile unsigned long *) 0xE002801C))

    // Memory Accelerator Module (MAM)

#define MAMCR          (*((volatile unsigned char *) 0xE01FC000))
#define MAMTIM         (*((volatile unsigned char *) 0xE01FC004))
#define MEMMAP         (*((volatile unsigned char *) 0xE01FC040))

    // Phase Locked Loop (PLL)

#define PLLCON         (*((volatile unsigned char *) 0xE01FC080))
#define PLLCFG         (*((volatile unsigned char *) 0xE01FC084))
#define PLLSTAT        (*((volatile unsigned short*) 0xE01FC088))
#define PLLFEED        (*((volatile unsigned char *) 0xE01FC08C))

    // VPB Divider

#define VPBDIV         (*((volatile unsigned char *) 0xE01FC100))

    // Power Control

#define PCON           (*((volatile unsigned char *) 0xE01FC0C0))

    // Peripheral Power Control

#define PCONP          (*((volatile unsigned long *) 0xE01FC0C4))
#   define PCI2C0           7   // I2C0
#   define PCRTC            9   // Real time clock

    // External Interrupts

#define EXTINT         (*((volatile unsigned char *) 0xE01FC140))
#define INTWAKE        (*((volatile unsigned char *) 0xE01FC144))
#define EXTMODE        (*((volatile unsigned char *) 0xE01FC148))
#define EXTPOLAR       (*((volatile unsigned char *) 0xE01FC14C))

    // Reset

#define RSID           (*((volatile unsigned char *) 0xE01FC180))

    // Code Security / Debugging

#define CSPR           (*((volatile unsigned char *) 0xE01FC184))

    // Timer 0

#define T0IR           (*((volatile unsigned long *) 0xE0004000))
#define T0TCR          (*((volatile unsigned long *) 0xE0004004))
#define T0TC           (*((volatile unsigned long *) 0xE0004008))
#define T0PR           (*((volatile unsigned long *) 0xE000400C))
#define T0PC           (*((volatile unsigned long *) 0xE0004010))
#define T0MCR          (*((volatile unsigned long *) 0xE0004014))
#define T0MR0          (*((volatile unsigned long *) 0xE0004018))
#define T0MR1          (*((volatile unsigned long *) 0xE000401C))
#define T0MR2          (*((volatile unsigned long *) 0xE0004020))
#define T0MR3          (*((volatile unsigned long *) 0xE0004024))
#define T0CCR          (*((volatile unsigned long *) 0xE0004028))
#define T0CR0          (*((volatile unsigned long *) 0xE000402C))
#define T0CR1          (*((volatile unsigned long *) 0xE0004030))
#define T0CR2          (*((volatile unsigned long *) 0xE0004034))
#define T0CR3          (*((volatile unsigned long *) 0xE0004038))
#define T0EMR          (*((volatile unsigned long *) 0xE000403C))
#define T0CTCR         (*((volatile unsigned long *) 0xE0004070))

    // Timer 1

#define T1IR           (*((volatile unsigned long *) 0xE0008000))
#define T1TCR          (*((volatile unsigned long *) 0xE0008004))
#define T1TC           (*((volatile unsigned long *) 0xE0008008))
#define T1PR           (*((volatile unsigned long *) 0xE000800C))
#define T1PC           (*((volatile unsigned long *) 0xE0008010))
#define T1MCR          (*((volatile unsigned long *) 0xE0008014))
#define T1MR0          (*((volatile unsigned long *) 0xE0008018))
#define T1MR1          (*((volatile unsigned long *) 0xE000801C))
#define T1MR2          (*((volatile unsigned long *) 0xE0008020))
#define T1MR3          (*((volatile unsigned long *) 0xE0008024))
#define T1CCR          (*((volatile unsigned long *) 0xE0008028))
#define T1CR0          (*((volatile unsigned long *) 0xE000802C))
#define T1CR1          (*((volatile unsigned long *) 0xE0008030))
#define T1CR2          (*((volatile unsigned long *) 0xE0008034))
#define T1CR3          (*((volatile unsigned long *) 0xE0008038))
#define T1EMR          (*((volatile unsigned long *) 0xE000803C))
#define T1CTCR         (*((volatile unsigned long *) 0xE0008070))

    // Timer 0/1 MCR Bits:

#define TMCR_MR0I           0x00000001  // 00 Interrupt on MR0
#define TMCR_MR0R           0x00000002  // 01 Reset on MR0
#define TMCR_MR0S           0x00000004  // 02 Stop on MR0

#define TMCR_MR1I           0x00000008  // 03 Interrupt on MR1
#define TMCR_MR1R           0x00000010  // 04 Reset on MR1
#define TMCR_MR1S           0x00000020  // 05 Stop on MR1

#define TMCR_MR2I           0x00000040  // 06 Interrupt on MR2
#define TMCR_MR2R           0x00000080  // 07 Reset on MR2
#define TMCR_MR2S           0x00000100  // 08 Stop on MR2

#define TMCR_MR3I           0x00000200  // 09 Interrupt on MR3
#define TMCR_MR3R           0x00000400  // 10 Reset on MR3
#define TMCR_MR3S           0x00000800  // 11 Stop on MR3


    // Pulse Width Modulator (PWM)

#define PWMIR          (*((volatile unsigned long *) 0xE0014000))
#define PWMTCR         (*((volatile unsigned long *) 0xE0014004))
#define PWMTC          (*((volatile unsigned long *) 0xE0014008))
#define PWMPR          (*((volatile unsigned long *) 0xE001400C))
#define PWMPC          (*((volatile unsigned long *) 0xE0014010))
#define PWMMCR         (*((volatile unsigned long *) 0xE0014014))
#define PWMMR0         (*((volatile unsigned long *) 0xE0014018))
#define PWMMR1         (*((volatile unsigned long *) 0xE001401C))
#define PWMMR2         (*((volatile unsigned long *) 0xE0014020))
#define PWMMR3         (*((volatile unsigned long *) 0xE0014024))
#define PWMMR4         (*((volatile unsigned long *) 0xE0014040))
#define PWMMR5         (*((volatile unsigned long *) 0xE0014044))
#define PWMMR6         (*((volatile unsigned long *) 0xE0014048))
#define PWMEMR         (*((volatile unsigned long *) 0xE001403C))
#define PWMPCR         (*((volatile unsigned long *) 0xE001404C))
#define PWMLER         (*((volatile unsigned long *) 0xE0014050))

    // Universal Asynchronous Receiver Transmitter 0 (UART0)

#define U0RBR          (*((volatile unsigned char *) 0xE000C000))
#define U0THR          (*((volatile unsigned char *) 0xE000C000))
#define U0IER          (*((volatile unsigned char *) 0xE000C004))
#define U0IIR          (*((volatile unsigned char *) 0xE000C008))
#define U0FCR          (*((volatile unsigned char *) 0xE000C008))
#define U0LCR          (*((volatile unsigned char *) 0xE000C00C))
#define U0LSR          (*((volatile unsigned char *) 0xE000C014))
#define U0SCR          (*((volatile unsigned char *) 0xE000C01C))
#define U0DLL          (*((volatile unsigned char *) 0xE000C000))
#define U0DLM          (*((volatile unsigned char *) 0xE000C004))
#define U0FDR          (*((volatile unsigned char *) 0xE000C028))
#define U0TER          (*((volatile unsigned char *) 0xE000C030))

    // Universal Asynchronous Receiver Transmitter 1 (UART1)

#define U1RBR          (*((volatile unsigned char *) 0xE0010000))
#define U1THR          (*((volatile unsigned char *) 0xE0010000))
#define U1IER          (*((volatile unsigned char *) 0xE0010004))
#define U1IIR          (*((volatile unsigned char *) 0xE0010008))
#define U1FCR          (*((volatile unsigned char *) 0xE0010008))
#define U1LCR          (*((volatile unsigned char *) 0xE001000C))
#define U1MCR          (*((volatile unsigned char *) 0xE0010010))
#define U1LSR          (*((volatile unsigned char *) 0xE0010014))
#define U1MSR          (*((volatile unsigned char *) 0xE0010018))
#define U1SCR          (*((volatile unsigned char *) 0xE001001C))
#define U1DLL          (*((volatile unsigned char *) 0xE0010000))
#define U1DLM          (*((volatile unsigned char *) 0xE0010004))
#define U1TER          (*((volatile unsigned char *) 0xE0010030))

    // I2C Interface 0

#define I20CONSET      (*((volatile unsigned char *) 0xE001C000))
#define I20CONCLR      (*((volatile unsigned char *) 0xE001C018))
#   define  I2C_AA      BV(2)
#   define  I2C_SI      BV(3)
#   define  I2C_STO     BV(4)
#   define  I2C_STA     BV(5)
#   define  I2C_EN      BV(6)

#define I20STAT        (*((volatile unsigned char *) 0xE001C004))
#define I20DAT         (*((volatile unsigned char *) 0xE001C008))
#define I20ADR         (*((volatile unsigned char *) 0xE001C00C))
#define I20SCLH        (*((volatile unsigned short*) 0xE001C010))
#define I20SCLL        (*((volatile unsigned short*) 0xE001C014))

    // I2C Interface 1 (Use same bit definitions as I2C0)

#define I21CONSET      (*((volatile unsigned char *) 0xE005C000))
#define I21CONCLR      (*((volatile unsigned char *) 0xE005C018))

#define I21STAT        (*((volatile unsigned char *) 0xE005C004))
#define I21DAT         (*((volatile unsigned char *) 0xE005C008))
#define I21ADR         (*((volatile unsigned char *) 0xE005C00C))
#define I21SCLH        (*((volatile unsigned short*) 0xE005C010))
#define I21SCLL        (*((volatile unsigned short*) 0xE005C014))

    /** @name I2C (I2C) Hardware State Definitions

        The following state numbers are returned by the I2C
        hardware and are read by the interrupt service routine
        to determine the state of the transfer.
    */
//@{
#define I2C_STS_START       0x08    ///< Start bit was transmitted
#define I2C_STS_RSTART      0x10    ///< Repeated Start was transmitted

#define I2C_STS_SLAWACK     0x18    ///< SLA/W written and ACKed
#define I2C_STS_SLAWNAK     0x20    ///< SLA/W written and NAKed

#define I2C_STS_WDATAACK    0x28    ///< Data byte written and ACKed
#define I2C_STS_WDATANAK    0x30    ///< Data byte written and NAKed

#define I2C_STS_SLARACK     0x40    ///< SLA/R written and ACKed
#define I2C_STS_SLARNAK     0x48    ///< SLA/R written and NAKed

#define I2C_STS_RDATAACK    0x50    ///< Data byte read and ACKed
#define I2C_STS_RDATANAK    0x58    ///< Data byte read and NAKed
//@}


    // SPI0 (Serial Peripheral Interface 0)

#define S0SPCR         (*((volatile uint16_t *) 0xE0020000))

#define S0SPSR         (*((volatile unsigned char *) 0xE0020004))
#   define SPIF                 7       // SPI Finished Flag

#define S0SPDR         (*((volatile uint16_t *) 0xE0020008))
#define S0SPCCR        (*((volatile unsigned char *) 0xE002000C))
#define S0SPTCR        (*((volatile unsigned char *) 0xE0020010))
#define S0SPTSR        (*((volatile unsigned char *) 0xE0020014))
#define S0SPTOR        (*((volatile unsigned char *) 0xE0020018))
#define S0SPINT        (*((volatile unsigned char *) 0xE002001C))

    // SSP Controller

#define SSPCR0         (*((volatile unsigned short* ) 0xE0068000))
#define SSPCR1         (*((volatile unsigned char * ) 0xE0068004))
#define SSPDR          (*((volatile unsigned short* ) 0xE0068008))
#define SSPSR          (*((volatile unsigned char * ) 0xE006800C))
#   define TFE              BV(0)
#   define TNF              BV(1)
#   define RNE              BV(2)
#   define RFF              BV(3)
#   define BSY              BV(4)

#define SSPCPSR        (*((volatile unsigned char * ) 0xE0068010))

#define SSPIMSC        (*((volatile unsigned char * ) 0xE0068014))
#   define RORIM            BV(0)
#   define RTIM             BV(1)
#   define RXIM             BV(2)
#   define TXIM             BV(3)

#define SSPRIS         (*((volatile unsigned char * ) 0xE0068018))
#   define RORRIS           BV(0)
#   define RTRIS            BV(1)
#   define RXRIS            BV(2)
#   define TXRIS            BV(3)

#define SSPMIS         (*((volatile unsigned char * ) 0xE006801C))
#   define RORMIS           BV(0)
#   define RTMIS            BV(1)
#   define RXMIS            BV(2)
#   define TXMIS            BV(3)

#define SSPICR         (*((volatile unsigned char * ) 0xE0068020))
#define SSPDMACR       (*((volatile unsigned char * ) 0xE0068024))

    // Real Time Clock

#define ILR            (*((volatile unsigned char *) 0xE0024000))
#define CTC            (*((volatile unsigned short*) 0xE0024004))
#define CCR            (*((volatile unsigned char *) 0xE0024008))
#define CIIR           (*((volatile unsigned char *) 0xE002400C))
#define AMR            (*((volatile unsigned char *) 0xE0024010))
#define CTIME0         (*((volatile unsigned long *) 0xE0024014))
#define CTIME1         (*((volatile unsigned long *) 0xE0024018))
#define CTIME2         (*((volatile unsigned long *) 0xE002401C))
#define SEC            (*((volatile unsigned char *) 0xE0024020))
#define MINUTE         (*((volatile unsigned char *) 0xE0024024))
#define HOUR           (*((volatile unsigned char *) 0xE0024028))
#define DOM            (*((volatile unsigned char *) 0xE002402C))
#define DOW            (*((volatile unsigned char *) 0xE0024030))
#define DOY            (*((volatile unsigned short*) 0xE0024034))
#define MONTH          (*((volatile unsigned char *) 0xE0024038))
#define YEAR           (*((volatile unsigned short*) 0xE002403C))
#define ALSEC          (*((volatile unsigned char *) 0xE0024060))
#define ALMIN          (*((volatile unsigned char *) 0xE0024064))
#define ALHOUR         (*((volatile unsigned char *) 0xE0024068))
#define ALDOM          (*((volatile unsigned char *) 0xE002406C))
#define ALDOW          (*((volatile unsigned char *) 0xE0024070))
#define ALDOY          (*((volatile unsigned short*) 0xE0024074))
#define ALMON          (*((volatile unsigned char *) 0xE0024078))
#define ALYEAR         (*((volatile unsigned short*) 0xE002407C))
#define PREINT         (*((volatile unsigned short*) 0xE0024080))
#define PREFRAC        (*((volatile unsigned short*) 0xE0024084))

    // A/D Converter 0 (AD0)

#define AD0CR          (*((volatile unsigned long *) 0xE0034000))
#define AD0DR          (*((volatile unsigned long *) 0xE0034004))

    // A/D Converter 1 (AD1)

#define AD1CR          (*((volatile unsigned long *) 0xE0060000))
#define AD1DR          (*((volatile unsigned long *) 0xE0060004))

    // D/A Converter

#define DACR           (*((volatile unsigned long *) 0xE006C000))


    // Watchdog

#define WDMOD          (*((volatile unsigned char *) 0xE0000000))
#define WDTC           (*((volatile unsigned long *) 0xE0000004))
#define WDFEED         (*((volatile unsigned char *) 0xE0000008))
#define WDTV           (*((volatile unsigned long *) 0xE000000C))

    // IAP Commands

typedef void (*IAP_TYPE)(unsigned int [],unsigned int[]);

#define IAP_ADDR        (IAP_TYPE)0x7FFFFFF1
#define IAP(x,y)        (*(IAP_ADDR))((x), (y))

