/****************************************************************//**

    @file std.h
    Standard Definitions for C Programs

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************/

//      *** MACROS ****

#define ISLOWER(c)      ((c) >= 'a' && (c) <= 'z')
#define TOUPPER(c)      (ISLOWER(c) ? (c)-'a'+'A' : (c))

#define CLIP(x, hi, lo) (((x) > (hi)) ? \
                                (hi) : (((x) < (lo)) ? (lo) : (x)))

#define ABS(x)              ((x) > 0? (x) : -(x))

#define MAX(a,b)            (((a) > (b)) ? (a) : (b))
#define MIN(a,b)            (((a) < (b)) ? (a) : (b))

#define FOREVER             for(;;)     // Loop until break
#define TILBREAK                for(;;)     // Loop until break

//lint -emacro(722,LOOP_TIL_true)   // Suspicious semicolon.
//#define LOOP_TIL_true(x)    while(!(x)) // Loop til (x) is true.
#define LOOP_TIL_true(x)    while((x) == 0) // Loop til (x) is true.

#define BV(x)                   ((unsigned)1 << (x))    // 2^(x)

#define STRINGIFY(x)		#x

    // The macros below provide the ability to generate print
    // and parse formats that are keyed to the size of a buffer.
    // For example, if you have a word buffer that is Z_LENGTH
    // bytes long, you can generate a "%{x}w" format, where
    // {x} is the value in Z_LENGTH as follows:
    //
    //      GEN_FORMAT(w, Z_LENGTH)
    //
    // The _GEN_FORMAT() macro is a helper that is required to
    // render the stringification of the values properly.

#define GEN_FORMAT(type, value) _GEN_FORMAT(type, value)
#define _GEN_FORMAT(type,value) "%" #value #type


//****************************************************************

#ifdef DEBUG
#undef NDEBUG
#else
#define NDEBUG
#endif

//****************************************************************

//      *** GENERAL DEFINITIONS ****

#define Z_CMD_LINE             132     // Command line size
#define Z_WORD                  40     // Ascii word size
#define Z_LINE_BUF              80     // Buffered line.


    // Logical and Null Values:
    //lint -emacro(506, true)

#if 0
#ifndef false
#define false               0
#define true                (!false)
#endif
#endif

#ifndef NULL
#define NULL                ((void *)0)
#endif

