/****************************************************************//**

	@file new_dusi.h
	@author EAS/NLH/alp

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************/
//                MODULE ID DEFINITIONS
//***************************************************************

#define DUSI_ID(x)  (MODULE(MID_DUSI) + (x))

#define TID_DUSI        DUSI_ID(1)  // DUSI task ID

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                  SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Signal that the command entered via DTMF was processed OK.
     *
     * @oo slotParm1 Rx channel number receiving DTMF.
     * @oo slotParm2 Not used.
     */
DECLARE_SIGNAL(DUSI_CommandOK)


    /** Signal that the command entered via DTMF was processed
     * with an error.
     *
     * @oo slotParm1 Rx channel number receiving DTMF.
     * @oo slotParm2 Not used.
     */
DECLARE_SIGNAL(DUSI_CommandERR)

    /** Signal that the voice recorder should be armed as a result
     * of a DUSI command.
     *
     * @oo slotParm1 Rx channel number that should be armed.
     * @oo slotParm2 Not used.
     */
DECLARE_SIGNAL(DUSI_ArmRecorder)


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

//		*** PUBLIC FUNCTION PROTOTYPES ***

void DUSI_Reset(void);
void DUSI_Start(void);

void DUSI_UnsubscribeAll(void);

void DUSI_DTMFCharReceived(enum TAG_RX_CHAN chan, char c);

extern const struct TAG_VARIABLE_TABLE dusiVarTab[];

#ifdef UNIT_TEST
void DUSI_GetChar(void);
//void DUSI_Report(char c);
#endif


