/******************************************************************//**

    @file simplesm.h
    Simple state machine support, API header.

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/

#include "apex_types.h"

//  *** Module Number and Related Definitions ***

#define SSM_ID(x)   (MODULE(MID_SSM) + (x))

#define TID_SSM     SSM_ID(1)   // Voice task ID

    /// A function of the type that simple state machines use as
    /// a task.

typedef void ssm_task_t(
                unsigned event, uintptr_t data1, uintptr_t data2);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      *** PUBLIC FUNCTION PROTOTYPES ***

void SSM_Reset(void);

void SSM_Start(void);

void SSM_CreateTask(ssm_task_t *pTask, uint8_t tid);

void SSM_SendMsg(unsigned id);

void SSM_SendMsgExt(unsigned id, uintptr_t data1, uintptr_t data2);

void SSM_InitTimer(TMR *pTimer,
            unsigned event, uintptr_t data1, uintptr_t data2);

void SSM_StartTmr(TMR *pTimer, int duration);

void SSM_StopTmr(TMR *pTimer);

extern const struct TAG_VARIABLE_TABLE ssmVarTab[];

#ifdef APEX_H
#error You may not include apex.h in a simple state machine module. Include simplesm.h only.
#endif
