/******************************************************************//**

    @file ssm_idsm.h

    ID State Machine for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/
//             *** MODULE ID DEFINITIONS ****
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#define IDSM_ID(x)  (MODULE(MID_IDSM) + (x))

#define TID_IDSM        IDSM_ID(1)  // ID task ID

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                      SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Signal that it's time to do an ID.
     *
     * @oo slotParm1 Tx Channel number.
     * @oo slotParm2 Not used.
     */
DECLARE_SIGNAL(IDSM_ScheduleID)

    /** Signal that it's time to do an Opening ID.
     *
     * @oo slotParm1 Tx Channel number.
     * @oo slotParm2 Not used.
     */
DECLARE_SIGNAL(IDSM_ScheduleOpeningID)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

//      *** PUBLIC FUNCTION PROTOTYPES ***

void IDSM_Reset(void);
void IDSM_Start(void);

void IDSM_UnsubscribeAll(void);

void IDSM_NotifyAudioOn(enum TAG_TX_CHAN txChan);
void IDSM_NotifyAudioOff(enum TAG_TX_CHAN txChan);
void IDSM_NotifyTxTimeout(enum TAG_TX_CHAN txChan);
void IDSM_NotifyTxDisable(enum TAG_TX_CHAN chan);
void IDSM_NotifyDoneID(enum TAG_TX_CHAN txChan);
void IDSM_NotifyChangedID(enum TAG_TX_CHAN chan);


extern const struct TAG_VARIABLE_TABLE idsmVarTab[];

