/******************************************************************//**

	@file ssm_morse.h

	Morse Code Encoder for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/
    // Apex Task Related Definitions:

#define MORSE_ID(x) (MODULE(MID_MORSE) + (x))

#define TID_MORSE       MORSE_ID(1)     // Morse task ID

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                      SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Signal that the Morse module has completed
     * transmission.
     *
     * @oo slotParm1 Tx Channel number.
     * @oo slotParm2 Not used.
     */
DECLARE_SIGNAL(MORSE_Complete)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

//		*** GENERAL DEFINITIONS ***

#define MIN_MORSE_CODE_RATE	10		// Min = 10 wpm.
#define MAX_MORSE_CODE_RATE	20		// Max = 20 wpm.


//		*** PUBLIC FUNCTION PROTOTYPES ***

/**
	Power-up initialization of Morse Code task.

	@return void.
*/
void Morse_Reset(void);
void Morse_Start(void);

void MORSE_UnsubscribeAll(void);

/**
	Encode a string into Morse Code and send it via audio tones.

	@param chan: Selects between transmitter channels.
	@param level: The audio level with witch to output the
	    morse characters.
	@param pString: Null terminated C string containing ASCII
		characters to encode and send.
	@return void.
*/
void Morse_Encode(
        enum TAG_TX_CHAN chan, uint8_t level, char *pString);

void Morse_Stop(enum TAG_TX_CHAN chan);

/** Validate a character as part of the Morse set.
 */
bool Morse_IsMorse(char c);

#if 0
void Morse_Whisper(enum TAG_TX_CHAN chan);

void Morse_Shout(enum TAG_TX_CHAN chan);
#endif

extern const struct TAG_VARIABLE_TABLE morseVarTab[];

