/*********************************************************************/
/**
    @file ssm_satd.h
    API for Subaudible Tone Decoder for PSE-702.

    @author ALP

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

*/
//*********************************************************************
//                    *** DEFINITIONS ***
//*********************************************************************

//  *** CTCSS Detector Module Number and Related Definitions ***

#define SATD_ID(x)  (MODULE(MID_SATD) + (x))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    /** Number of entries in subaudible tone freqency table.

        This definition sets the length of the subaudible tone
        parameter array in satd.c.
        The maximum subaudible tone index that may be
        entered by the user is one less than the value in this
        definition.
        This definition must be changed if lines are added or removed
        from the table.
    */

#define SATD_FILTER_TABLE_SIZE  37  ///< Max number of CTCSS tones.

//*********************************************************************
//                           SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

DECLARE_SIGNAL(SATD_Locked)
DECLARE_SIGNAL(SATD_Unlocked)
DECLARE_SIGNAL(SATD_SigLevel)
DECLARE_SIGNAL(SATD_ZXing)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                  *** PUBLIC FUNCTION PROTOTYPES ***
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Initialize the subaudible tone decoder upon reset.
    */
void SATD_Reset(void);

void SATD_Start(void);

void SATD_UnsubscribeAll(void);

    // Return a frequency in 1/10 Hz given an index

unsigned SATD_TranslateIndex(unsigned index);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Turn the decoders off.
        @param rxChan: Receiver channel number.
        @return Nothing.

        This function disables all decoders for the indicated
        receiver channel.
    */
void SATD_DisableDecoder(enum TAG_RX_CHAN rxChan);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Turn the decoders on.

        @param rxChan: Receiver channel number.
        @return Nothing.

        This function enables all decoders for the indicated
        receiver channel.
    */
void SATD_EnableDecoder(enum TAG_RX_CHAN rxChan);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
/** Load a new set of detector parameters into a CTCSS decoder.

    This function is called on reset and whenever the parameters
    are thought to have changed (such as via EEP_SetPathRx()).

    @param rxChan: An enum identifying the receiver channel
        to which parameters should be loaded.

    @return: Nothing.
*/

void SATD_LoadDecoderParms(enum TAG_RX_CHAN rxChan);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void SATD_SampleSinkChan1_ISR(int16_t x_sub_n);


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void SATD_SampleSinkChan2_ISR(int16_t x_sub_n);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

extern const struct TAG_VARIABLE_TABLE satdVarTab[];

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                          END
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

