/******************************************************************//**

    @file ssm_sate.h

    Subaudible Tone Encoder State Machine for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/

#define SATE_ID(x)  (MODULE(MID_SATE) + (x))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                      SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Signal that the subaudible tone encoder has completed
     * transmission.
     *
     * @oo slotParm1 Tx Channel number.
     * @oo slotParm2 Not used.
     */
DECLARE_SIGNAL(SATE_TurnedOff)

/** Signal that the subaudible tone encoder has started transmission.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(SATE_TurnedOn)

/** Signal that the subaudible tone encoder has started a
 * reverse burst.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(SATE_RevBurst)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#define MAX_SATE_INDEX      36      ///< Maximum SATE freq index.

//      *** DATA STRUCTURE DEFINITIONS ***

enum TAG_SATE_MODE
{
    SATE_MODE_RUS,
    SATE_MODE_PTT,
    SATE_MODE_MAX
};

    // Convert numbers 1 and 2 to modes RUS and PTT:

#define ORDINAL_2_SATE_MODE(x)  ((enum TAG_SATE_MODE)((x)-1))
#define SATE_MODE_2_ORDINAL(x)  ((enum TAG_SATE_MODE)((x)+1))
#define VALID_SATE_MODE(x)      ((x)<SATE_MODE_MAX)

//      *** PUBLIC FUNCTION PROTOTYPES ***

void SATE_Reset(void);
void SATE_Start(void);

void SATE_UnsubscribeAll(void);

void SATE_On(enum TAG_TX_CHAN txChan);
void SATE_Off(enum TAG_TX_CHAN txChan);
void SATE_UpdateFrequency(enum TAG_TX_CHAN txChan);

extern const struct TAG_VARIABLE_TABLE sateVarTab[];

