/******************************************************************//**

    @file ssm_txsm.h

    Transmitter State Machine for PSE-702

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/
//           *** MODULE ID DEFINITIONS ****
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#define TXSM_ID(x)  (MODULE(MID_TXSM) + (x))

#define TID_TXSM        TXSM_ID(1)  // Transmitter task ID

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                      SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
/** Signal that the Transmit State Machine is turning
 * a transmitter off.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(TXSM_TxOff)

/** Signal that the Transmit State Machine is turning a
 * transmitter on.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(TXSM_TxOn)

/** Signal that the Transmit State Machine wants the
 * audio generator to start sending courtesy beeps, IDs,
 * etc.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(TXSM_StartGeneratedAudio)

/** Signal that the Transmit State Machine wants to have
 * a courtesy beep scheduled to occur after the user audio
 * is finished.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
//DECLARE_SIGNAL(TXSM_ScheduleCB)

/** Signal that the Transmit State Machine has timed out
 * a transmit channel.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(TXSM_TxTimeout)

#if 0
/** Signal that the Transmit State Machine wants the sub-
 * audible tone encoder to stop transmitting.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(TXSM_StopSATE)
#endif

/** Signal that the Transmit State Machine has finished all
 * transmit jobs and can now turn off.  It will wait for SATE
 * off before actually shutting off the transmitter in case
 * there is a reverse burst.
 *
 * @oo slotParm1 Tx Channel number.
 * @oo slotParm2 Not used.
 */
DECLARE_SIGNAL(TXSM_TxComplete)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      *** PUBLIC FUNCTION PROTOTYPES ***

void TXSM_Reset(void);
void TXSM_Start(void);

void TXSM_UnsubscribeAll(void);

void TXSM_NotifyUserAudioOn(enum TAG_TX_CHAN chan);
void TXSM_NotifyUserAudioOff(enum TAG_TX_CHAN chan);

void TXSM_NotifyGeneratedAudioOn(enum TAG_TX_CHAN chan);
void TXSM_NotifyGeneratedAudioOff(enum TAG_TX_CHAN chan);

void TXSM_DTMFMute(enum TAG_TX_CHAN txChan);

void TXSM_NotifySATE_TurnedOn(enum TAG_TX_CHAN chan);
void TXSM_NotifySATE_TurnedOff(enum TAG_TX_CHAN chan);

void TXSM_NotifyTxTimeoutCleared(enum TAG_TX_CHAN chan);

void TXSM_NotifyTxDisable(enum TAG_TX_CHAN txChan);
void TXSM_NotifyTxEnable(enum TAG_TX_CHAN txChan);

    /** Turn on PTT on a selected channel:
     *
     */
void TXSM_TurnPTTOn(enum TAG_TX_CHAN txChan);

    /** Turn off PTT on a selected channel:
     *
     */
void TXSM_TurnPTTOff(enum TAG_TX_CHAN txChan);


extern const struct TAG_VARIABLE_TABLE txsmVarTab[];

