/****************************************************************
*                                                               *
*   susi_cmds.h: Serial User Interface Command Header           *
*                                                               *
    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.
*                                                               *
****************************************************************/

/***************************************************************/
//    *** MACROS ****

    /// SUSI Module ID

#define SUSI_ID(x)   (MODULE(MID_SUSI) + (x))

    /// State Machine Event IDs:

enum TAG_SUSI_EVIDS
{
    EV_SUSI_RXDATA = SUSI_ID(0),    ///< Got a character.
    EV_SUSI_CHAR_TIMEOUT,           ///< Character timeout
    EV_SUSI_SESS_TIMEOUT,           ///< Session timeout
    EV_SUSI_SIGNON,                 ///< Print signon message.
    EV_SUSI_TEST_TONE_DENIED,       ///< Cannot do a test tone
    EV_SUSI_TEST_TONE_GRANTED,      ///< Test tone OK.
    EV_SUSI_RECORD_STARTED,         ///< Started recording
    EV_SUSI_RECORD_OVERFLOW,        ///< Voice recording overflow.
    EV_SUSI_RECORD_COMPLETE,        ///< Voice recording complete OK.
    EV_SUSI_RECORD_CHAN_BUSY,       ///< Attempt to record on busy chan.
    EV_SUSI_EEP_WRITE_TIMEOUT,      ///< Time to write EEP.
    EV_SUSI_LEVEL_TIMEOUT,          ///< Time to sample the Rx level.
    EV_SUSI_MAX
};

    // State Functions

void SUSI_Transition_State_Interactive(void); // A

void SUSI_Transition_State_XModemReceive(void); // B

void SUSI_Transition_State_Start(void);   // C

void SUSI_Transition_State_TestToneRequest(void); // D

void SUSI_Transition_State_VoiceRecord(void); // E

void SUSI_Transition_State_StatusList(void); // F

void SUSI_Transition_State_HelpList(void);    // G

void SUSI_Transition_State_RxLevelAuto(
                            enum TAG_RX_CHAN chan); // H

void SUSI_SetState(void (*pNewState)(
        enum TAG_SUSI_EVIDS id, uintptr_t data1, uintptr_t data2),
        char c);

void SUSI_InitTimer(TMR *pTmr, unsigned id);

    // Receiver Commands:

void SUSI_RxCommandsReset(void);

uint8_t SUSI_CmdSetBeepTone(const char *pLine);
void SUSI_StatusSetBeepTone(void);

uint8_t SUSI_CmdSetBeepDelay(const char *pLine);
void SUSI_StatusSetBeepDelay(void);

uint8_t SUSI_CmdSetBeepMode(const char *pLine);
void SUSI_StatusSetBeepMode(void);

uint8_t SUSI_CmdSetBeepCharacter(const char *pLine);
void SUSI_StatusSetBeepCharacter(void);

uint8_t SUSI_CmdSetBeepLevel(const char *pLine);
void SUSI_StatusSetBeepLevel(void);

uint8_t SUSI_CmdSetRxFlags(const char *pLine);
void SUSI_StatusSetRxFlags(void);

uint8_t SUSI_CmdSetRxLevel(const char *pLine);
void SUSI_StatusSetRxLevel(void);

uint8_t SUSI_CmdSetRxLevelAuto(const char *pLine);

uint8_t SUSI_CmdSetCTCSSDecodeIndex(const char *pLine);
void SUSI_StatusCTCSSDecodeIndex(void);

uint8_t SUSI_CmdSetFilterIndex(const char *pLine);
void SUSI_StatusSetFilterIndex(void);

    // Transmitter Commands

uint8_t SUSI_CmdSetMorseRate(const char *pLine);
void SUSI_StatusSetMorseRate(void);

uint8_t SUSI_CmdSetMorseLevel(const char *pLine);
void SUSI_StatusSetMorseLevel(void);

uint8_t SUSI_CmdSetIDString(const char *pLine);
void SUSI_StatusSetIDString(void);

uint8_t SUSI_CmdSetTimers(const char *pLine);
void SUSI_StatusSetTimers(void);

uint8_t SUSI_CmdSetTxFlags(const char *pLine);
void SUSI_StatusSetTxFlags(void);

uint8_t SUSI_CmdSetTxLevel(const char *pLine);
void SUSI_StatusSetTxLevel(void);

uint8_t SUSI_CmdSetCTCSSEncodeIndex(const char *pLine);
void SUSI_StatusCTCSSEncodeIndex(void);

uint8_t SUSI_CmdSetCTCSSEncodeMode(const char *pLine);
void SUSI_StatusCTCSSEncodeMode(void);

uint8_t SUSI_CmdVoiceID(const char *pLine);
void SUSI_StatusVoiceID(void);

uint8_t SUSI_CmdPlayID(const char *pLine);

    //  System Commands

#if EMULATOR == 0
uint8_t SUSI_CmdReset(const char *pLine);
#endif

uint8_t SUSI_CmdDefaults(const char *pLine);

uint8_t SUSI_CmdSetDTMFPwd(const char *pLine);
void SUSI_StatusSetDTMFPwd(void);

uint8_t SUSI_CmdEnableDTMFPassword(const char *pLine);
void SUSI_StatusEnableDTMFPassword(void);

uint8_t SUSI_CmdVersion(const char *pLine);

uint8_t SUSI_CmdSetDTMFCommandChar(const char *pLine);
void SUSI_StatusSetDTMFCommandChar(void);

uint8_t SUSI_CmdScheduleAnno(const char *pLine);
void SUSI_StatusScheduleAnno(void);

uint8_t SUSI_CmdEnableAnno(const char *pLine);
void SUSI_StatusEnableAnno(void);

uint8_t SUSI_CmdSetRxTxPath(const char *pLine);
void SUSI_StatusSetRxTxPath(void);

uint8_t SUSI_CmdDeleteAnno(const char *pLine);
uint8_t SUSI_CmdDeletePath(const char *pLine);
uint8_t SUSI_CmdFileRecord(const char *pLine);
uint8_t SUSI_CmdFileDelete(const char *pLine);
uint8_t SUSI_CmdFileFormat(const char *pLine);
uint8_t SUSI_CmdFileSystemStatus(const char *pLine);
uint8_t SUSI_CmdPlayFile(const char *pLine);

uint8_t SUSI_CmdXModem(const char *pLine);


    // Debug Commands

void SUSI_DebugReset(void);

uint8_t SUSI_CmdWriteFlashPage(const char *pline);
uint8_t SUSI_CmdReadFlashPage(const char *pline);
uint8_t SUSI_CmdWriteFlashByte(const char *pline);
uint8_t SUSI_CmdReadFlashByte(const char *pline);
uint8_t SUSI_CmdReadFlashSerialNumber(const char *pline);

uint8_t SUSI_CmdEraseFlash(const char *pline);
uint8_t SUSI_CmdReadFlashStatus(const char *pline);
uint8_t SUSI_CmdWriteFlashStatus(const char *pline);
uint8_t SUSI_CmdFillFlashBuf(const char *pLine);
uint8_t SUSI_CmdDisplayFlash(const char *pLine);
uint8_t SUSI_CmdUnprotectFlash(const char *pLine);

uint8_t SUSI_CmdFileRead(const char *pLine);
uint8_t SUSI_CmdFileAppend(const char *pLine);

uint8_t SUSI_CmdSetRTC(const char *pLine);
uint8_t SUSI_CmdReadRTC(const char *pLine);

uint8_t SUSI_CmdDisplayActivePaths(const char *pLine);
uint8_t SUSI_CmdArmBreakpoint(const char *pLine);
uint8_t SUSI_CmdDisplaySRAM(const char *pLine);
uint8_t SUSI_CmdSelectModule(const char *pLine);
uint8_t SUSI_CmdDisplayVariable(const char *pLine);
uint8_t SUSI_CmdDownload(const char *pLine);


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                          END
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

