/******************************************************************//**

    @file tp_application.h

    Firmware Testpoints for Testing and Troubleshooting

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

    @par Description

    This file contains conditional compilation flags that enable
    firmware testpoints.  The testpoints may be used with a scope
    to trace firmware execution and to measure critical firmware
    parameters such as interrupt latency.

    To enable a testpoint, change the value from 0 to 1.

**********************************************************************/
    /// Define a firmware version number:

    /// 130527 3.1.2: Test version w/ CTCSS reject filter
    /// enable/disable.

    /// 130617 3.2: Sent to S. Fritts for evaluation.  He reports
    /// that he is unable to command the controller via touch-
    /// tones using one of his radios.

    /// 130706 3.2.1 Setting the DTMF pre-filter back to a cuttoff
    /// of 400 Hz.

    /// 130729 3.2.2 Fixed DTMF problem that S. Fritts was having.
    /// It seems that DTMF tones should NOT be deemphasized when
    /// deemphasis is enabled on a channel.

    /// 131001 3.3 Added rxla command.  Changed txl to use 100
    /// levels in logrithmic pattern.

#define FIRMWARE_VERSION             "3.3"

    /// Set to 1 to make codec interrupt call astream test
    /// filter. Useful to unit test different filters.
#define TP_FILTER_UNIT_TEST             0

    /// Print important structure sizes out on the debug
    /// terminal at startup if this flag is non-zero.

#define TP_STRUCTURE_SIZES              0

    /// Assert a fimrware testpoint when interrupts are
    /// disabled.  Define as, for example,  O_FTP1_H();
    /// (Put a semicolon after the definition here,not
    /// at the point of use).

#define TP_INTERRUPT_LATENCY_DI_SIGNAL

    /// De-assert a fimrware testpoint when interrupts are
    /// enabled.  Define as, for example,  O_FTP1_L();
    /// (Put a semicolon after the definition here,not
    /// at the point of use).

#define TP_INTERRUPT_LATENCY_EI_SIGNAL

    /// Halt execution after an ASSERT if non-zero

#define TP_ASSERT_TRAP_ENABLE           1

    /// Define this as 'O_FTP0_STROBE()', for example, to strobe
    /// FTP0 when reset is complete.  Any such
    /// strobe or output command may be used. Leave blank to
    /// disable.  Put the semicolon after the definition here,
    /// not where the macro is used.

#define TP_RESET_COMPLETE

    /** Set the following definition to 1 to enable the audio
        delay feature.
    */
#define TP_AUDIO_DELAY_ENABLE           0

#if 0
    /// Define this as non-zero to obtain access to module
    /// variable tables via susi commands.

#define TP_ENABLE_VARTABS               1
#endif

    /// Define this as 'O_FTP0_STROBE()', for example, to strobe
    /// FTP0 when the subaudible tone detector starts.  Any such
    /// strobe or output command may be used. Leave blank to
    /// disable.  Put the semicolon after the definition here,
    /// not where the macro is used.

#define TP_SATD_ON

    /// Define this as 'O_FTP0_STROBE()', for example, to strobe
    /// FTP0 when the subaudible tone detector locks.  Any such
    /// strobe or output command may be used. Leave blank to
    /// disable.  Put the semicolon after the definition here,
    /// not where the macro is used.

#define TP_SATD_LOCK

    /// Define this as 'O_FTP0_STROBE()', for example, to strobe
    /// FTP0 when the subaudible tone detector unlocks.  Any such
    /// strobe or output command may be used. Leave blank to
    /// disable.  Put the semicolon after the definition here,
    /// not where the macro is used.

#define TP_SATD_UNLOCK

//*********************************************************************
//          INTERRUPT DURATION FLAGS
//
//  The following definitions allow the programmer to assert a
//  firmware testpoint while a specific interrupt is executing.
//  This can be used to measure interrupt duration or determine
//  if some event occurs while the interrupt is executing.
//
//  To use these flags, simply define a test point function for
//  the TP_<ISR>_INTERRUPT_START and TP_<ISR>_INTERRUPT_END
//  definitions.  To turn off the testpoint signal, leave the
//  definition blank.
//
//  For example, to raise FTP0 during codec interrupts, define
//  the following:
//
//  #define TP_CODEC_INTERRUPT_START    O_FTP0_H();
//  #define TP_CODEC_INTERRUPT_END      O_FTP0_L();
//
//  Note that the semicolon is included in the definition and is
//  not present where the macro is used.
//
//*********************************************************************

    /// Assert a firmware testpoint around the spi
    /// interrupt.
#define TP_SPI_INTERRUPT_START
#define TP_SPI_INTERRUPT_END

    /// Assert a firmware testpoint around the console
    /// UART interrupt.
#define TP_UART0_INTERRUPT_START
#define TP_UART0_INTERRUPT_END

    /// Assert a firmware testpoint around the Timer 0
    /// (system tick) interrupt.
#define TP_TMR0_INTERRUPT_START
#define TP_TMR0_INTERRUPT_END

    /// Assert a firmware testpoint around the PWM
    /// interrupt.
#define TP_PWM_INTERRUPT_START
#define TP_PWM_INTERRUPT_END

    /// Assert a firmware testpoint around the codec
    /// interrupt.
#define TP_CODEC_INTERRUPT_START
#define TP_CODEC_INTERRUPT_END

    /// Assert a firmware testpoint around the reading
    /// of a voice file page.
#define TP_VOICE_READPAGE_START
#define TP_VOICE_READPAGE_END

    /// Strobe an output when a voice sample is taken.
#define TP_VOICE_GETSAMPLE_STROBE

//*********************************************************************
//          TASK DURATION FLAGS
//
//  The following definitions allow the programmer to assert a
//  firmware testpoint while a task is executing.  This can be
//  used to measure task completion times or determine if some
//  event occurs while the task is executing.
//
//  To use this feature, define TP_TASK_MONITOR_ID to the ID
//  number that is passed to the task's TCB when the task is
//  created.  Then define TP_TASK_MONITOR_START and TP_TASK_MONITOR_END
//  to some command to set and clear a firmware testpoint.
//
//  For example, to monitor the duration of the console task on FTP0,
//  use the following definitions:
//
//  #define TP_TASK_MONITOR_ID      TID_CON
//  #define TP_TASK_MONITOR_START   O_FTP0_H();
//  #define TP_TASK_MONITOR_END     O_FTP0_L();
//
//  Note that the semicolon is appended to the macro definition,
//  not to where the macro is actually used.
//
//  To turn off these definitions, set TP_TASK_MONITOR_ID to zero.
//
//*********************************************************************

#define TP_TASK_MONITOR_ID              0
#define TP_TASK_MONITOR_START
#define TP_TASK_MONITOR_END

//*********************************************************************
//          STATE CHANGE DISPLAY FLAGS
//
//  The way these should work is as follows:  If the value is
//  zero, the flags are disabled.  If the value is odd, the
//  flags will display upper case.  If the value is even, the
//  flags will display lower case.  Values 1 and 2 pertain
//  to channel 1, and values 3 and 4 pertain to channel 2.
//
//*********************************************************************

#define STATE_FLAGS_OFF             0
#define STATE_FLAGS_UPPER_CH1       1
#define STATE_FLAGS_LOWER_CH1       2
#define STATE_FLAGS_UPPER_CH2       3
#define STATE_FLAGS_LOWER_CH2       4

    /** If TP_SUSI_STATE_FLAGS is non-zero, susi will
        print its state changes on the debug console.
    */
#define TP_SUSI_STATE_FLAGS             STATE_FLAGS_OFF

    /** If TP_TTCON_STATE_FLAGS is non-zero, ttcon will
        print its state changes on the debug console.
    */
#define TP_TTCON_STATE_FLAGS            STATE_FLAGS_OFF

    /** If TP_CEN_RX_STATE_FLAGS is non-zero, the Central task will
        print its receiver state changes on the debug console.
    */
#define TP_CEN_RX_STATE_FLAGS           STATE_FLAGS_OFF
#define TP_CEN_RX_STATE_OFFSET          0

    /** If TP_CEN_SQ_STATE_FLAGS is non-zero, the Central task will
        print its squelch state changes on the debug console.
    */
#define TP_CEN_SQ_STATE_FLAGS           STATE_FLAGS_OFF
#define TP_CEN_SQ_STATE_OFFSET          0

    /** If TP_CEN_TX_STATE_FLAGS is non-zero, the Central task will
        print its transmitter state changes on the debug console.
    */
#define TP_CEN_TX_STATE_FLAGS           STATE_FLAGS_OFF
#define TP_CEN_TX_STATE_OFFSET          12

    /** If TP_SQU_STATE_FLAGS is non-zero, the carrier squelch
     * detector task will print its state changes on the debug
     * console.
    */
#define TP_SQU_STATE_FLAGS              STATE_FLAGS_OFF

    /** If TP_TXSM_STATE_FLAGS is non-zero, the Tx task will
        print its state changes on the debug console.
    */
#define TP_TXSM_STATE_FLAGS             STATE_FLAGS_OFF


    /** If TP_IDSM_STATE_FLAGS is non-zero, the ID task will
        print its state changes on the debug console.
    */
#define TP_IDSM_STATE_FLAGS             STATE_FLAGS_OFF

    /** If TP_AUD_STATE_FLAGS is non-zero, the Audio task will
        print its state changes on the debug console.
    */
#define TP_AUD_STATE_FLAGS              STATE_FLAGS_OFF

    /** If TP_SATE_STATE_FLAGS is non-zero, the Sub-Audible
     * tone encoder state machine task will print its state
     * changes on the debug console.
    */
#define TP_SATE_STATE_FLAGS             STATE_FLAGS_OFF

    /** If TP_SATD_STATE_FLAGS is non-zero, the Sub-Audible
     * tone decoder state machine task will print its state
     * changes on the debug console.
    */
#define TP_SATD_STATE_FLAGS             STATE_FLAGS_OFF

    /** If TP_DUSI_STATE_FLAGS is non-zero, the DTMF user
     * interface state machine task will print its state
     * changes on the debug console.
    */
#define TP_DUSI_STATE_FLAGS             STATE_FLAGS_OFF

    /** If TP_DTMF_STATE_FLAGS is non-zero, the DTMF
     * detector state machine task will print its state
     * changes on the debug console.
    */
#define TP_DTMF_STATE_FLAGS             STATE_FLAGS_OFF

    /** If TP_MORSE_STATE_FLAGS is non-zero, the Morse
     * Code generator state machine task will print its
     * state changes on the debug console.
    */
#define TP_MORSE_STATE_FLAGS            STATE_FLAGS_OFF

    /** If TP_XMODM_STATE_FLAGS is non-zero, the X-Modem
     * state machine will print its state changes on the
     * debug console.
    */
#define TP_XMODM_STATE_FLAGS            STATE_FLAGS_OFF

    /** If TP_SETUP_STATE_FLAGS is non-zero, the Setup
     * state machine will print its state changes on the
     * debug console.
    */
#define TP_SETUP_STATE_FLAGS            STATE_FLAGS_OFF

    /** If TP_VOI_STATE_FLAGS is non-zero, the Voice
     * state machine will print its state changes on the
     * debug console.
    */
#define TP_VOI_STATE_FLAGS              STATE_FLAGS_OFF

//*********************************************************************
//                          END OF FILE
//*********************************************************************
