/****************************************************************//**

    @file user_parms.h

    User Parameter Definitinos for PSE-702

    @defgroup eep User Parameter Structures

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************/


//***************************************************************
//          *** Data Structure Definitions ***
//***************************************************************

    /// The user parameter revision number is incremented each
    /// time the structure is changed.  This allows us to convert
    /// the user's parameters from a previous structure to a new
    /// structure without invalidating the user's parms and forcing
    /// him to re-enter them.

#define EEP_STRUCT_REVISION     1


//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      Parameter Structure, Current
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    // History:
    //
    // Revision     Changes
    //      1       Added CTCSS rejection filter selections.

struct TAG_EEP_RX_CHAN
{
    uint16_t flags;             ///< See eep.h for defines
    uint8_t rxLevel;            ///< Receiver channel digital pot
    uint8_t satdIndex;          ///< CTCSS frequency index
    uint8_t deemphIndex;        ///< Deemphasis filter index
    uint8_t rejectIndex;        ///< Reject filter index
    eep_courtsey_beep_t beep;   ///< Courtsey beep
};

struct TAG_EEP_TX_CHAN
{
    // Word aligned items

    uint16_t flags;             ///< See eep.h for defines

    uint16_t hangTime;          ///< Length of hang time in mS
    uint16_t txTimeout;         ///< Length of Tx timeout in Sec
    uint16_t idTime;            ///< Length of ID timer in Sec.
    uint16_t morseFreq;         ///< Morse tone frequency

    // Byte aligned items

    char idString[Z_WORD];      ///< CW ID string
    enum TAG_SATE_MODE sateMode; ///< CTCSS encoder mode
    uint8_t sateIndex;          ///< CTCSS frequency index
    uint8_t txLevelIndex;       ///< Tx channel gain/atten index
    uint8_t vidFileNum;         ///< Voice ID File Number
    uint8_t cbLevel;            ///< Courtesy beep audio level
    uint8_t morseLevel;         ///< Morse code audio Level
    uint8_t morseRate;          ///< Morse rate in words per minute
};

struct TAG_EEP_USER_PARMS
{
    uint16_t crc;       ///< Checksum of EEPROM data
    uint16_t revision;  ///< Cyclic revision number.

    uint16_t systemFlags;
    char passWord[Z_DTMF_PASSWORD]; ///< DTMF password string
    char dtmfCmdChar;           ///< Char that prefixes DTMF cmds

    struct TAG_EEP_RX_CHAN rxChanStruct[MAX_RXCHAN];
    struct TAG_EEP_TX_CHAN txChanStruct[MAX_TXCHAN];
    struct TAG_EEP_PATH pathStruct[MAX_PATHS];

    eep_anno_t annoStruct[MAX_ANNO];
};

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      Parameter Structure, Revision 0
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#define EEP_RX_FLAG_DEEMPH_0    BV(2)   ///< Deemphasis enable
#define EEP_RX_FLAG_SATR_0      BV(4)   ///< Enable tone reject filter

struct TAG_EEP_RX_CHAN_0
{
    uint16_t flags;             ///< See eep.h for defines
    uint8_t rxLevel;            ///< Receiver channel digital pot
    uint8_t satdIndex;          ///< CTCSS frequency index
    uint8_t deemphIndex;        ///< Deemphasis filter index
    eep_courtsey_beep_t beep;   ///< Courtsey beep
};

struct TAG_EEP_USER_PARMS_0
{
    uint16_t crc;       ///< Checksum of EEPROM data
    uint16_t revision;  ///< Cyclic revision number.

    uint16_t systemFlags;
    char passWord[Z_DTMF_PASSWORD]; ///< DTMF password string
    char dtmfCmdChar;           ///< Char that prefixes DTMF cmds

    struct TAG_EEP_RX_CHAN_0 rxChanStruct[MAX_RXCHAN];
    struct TAG_EEP_TX_CHAN txChanStruct[MAX_TXCHAN];
    struct TAG_EEP_PATH pathStruct[MAX_PATHS];

    eep_anno_t annoStruct[MAX_ANNO];
};

//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                      END
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
