/******************************************************************//**

    @file voice.h
    Voice store and playback for PSE-702.

    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************/
//  *** Voice Module Number and Related Definitions ***

#define VOI_ID(x)   (MODULE(MID_VOI) + (x))

#define TID_VOI     VOI_ID(1)   // Voice task ID

    // Voice channel IDs:

enum TAG_VOI_CHAN
{
    VOI_CHAN_1 = 0,     ///< Voice Channel for Tx1 ID
    VOI_CHAN_2,         ///< Voice Channel for Tx2 ID
    MAX_VOI_CHAN        ///< Number of channels
};

#define VALID_VOI_CHAN(x)  ((enum TAG_VOI_CHAN)(x) < MAX_VOI_CHAN)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//                  SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    /// Tell subscriber that the voice module is done with the
    /// last record/playback request.
DECLARE_SIGNAL(VOI_Signal_RecordComplete)
DECLARE_SIGNAL(VOI_Signal_PlaybackComplete)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//      *** PUBLIC FUNCTION PROTOTYPES ***

void VOI_Reset(void);
void VOI_Start(void);

void VOI_RecordFile(enum TAG_VOI_CHAN chan, unsigned file);
void VOI_StopRecording(enum TAG_VOI_CHAN chan);

void VOI_PlaybackFile(enum TAG_VOI_CHAN chan, unsigned file);
void VOI_StopPlayback(enum TAG_VOI_CHAN chan);

void VOI_UnsubscribeAllSignals(void);

int16_t VOI_GetSample_ISR(enum TAG_VOI_CHAN chan);
void VOI_PutSample_ISR(enum TAG_VOI_CHAN chan, int16_t sample);

extern const struct TAG_VARIABLE_TABLE voiceVarTab[];
