/***************************************************************
*                                                               *
*   xmodem.h: X-Modem State Machine for PSE-702                 *
*                                                               *
    @copyright
        Copyright 2018 by Pion and Simon Electronics.  All
        rights reserved.

    This file is part of the PSE-702 project

    The PSE-702 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The PSE-702 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file.  If not, see <http://www.gnu.org/licenses/>.
*                                                               *
****************************************************************/
//                  SIGNALS
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#define XMODEM_PAYLOAD_Z       128

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

    /// Tell subscriber done with X-Modem session

DECLARE_SIGNAL(XMODM_Signal_Done)

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

void XMODM_Reset(void);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    /** Start an X-Modem Session
     *
     * This function is called by the x-modem client module to
     * begin an x-modem session.  The client should feed messages
     * received at the console mailbox to the x-modem state
     * machine after calling this function.
     *
     * @param pFunc A pointer to a function which shall be called
     * when a buffer of data has been received via x-modem
     * protocol.
     *
     */
void XMODM_StartSession(void (*pFunc)(char *pBuff));

    /** The X-Modem State Machine
     *
     * The X-Modem state machine follows the protocol to receive
     * data via x-modem.  The client should call this function
     * whenever a message is received in the console's mailbox.
     */
void XMODM_StateMachine(unsigned id, uintptr_t data1);

void XMODM_UnsubscribeAllSignals(void);

extern const struct TAG_VARIABLE_TABLE xmodmVarTab[];

